﻿using KSProgram;
using KSProgram.Data;
using KSProgram.Dokument;
using KSProgram.Extensions;
using KSProgram.Pozadavky;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;

namespace KSPortal.Dotazniky
{
    public partial class WebMethods : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        [WebMethod]
        public static DataResult GetLokace()
        {
            List<DynamicDataObject> data = GetSouradniceZamestnancu();

            var definice = new
            {
                os_cislo = 0m,
                jmeno = "",
                adresa = "",
                location = new double[] { 0, 0 },
                kraj = "",
                mesto = "",
            };

            return new DataResult
            {
                Data = from d in data.ToEnumarable(definice)
                       select (object)d,
                Count = data.Count
            };
        }

        [WebMethod]
        public static DataResult GetMesta()
        {
            List<DynamicDataObject> data = GetSouradniceZamestnancu();

            return new DataResult
            {
                Data = (from d in data
                        group d by d.Get("mesto")?.ToString()
                        into grouped
                        select ((object)new
                        {
                            item = grouped.Key,
                            value = grouped.Count(),
                            location = new double[2] { grouped.Average(x => ((double[])x.Get("location"))[0]), grouped.Average(x => ((double[])x.Get("location"))[1]) },
                        })).ToList(),
                Count = data.Count
            };
        }

        public static List<DynamicDataObject> GetSouradniceZamestnancu()
        {
            string cachekey = "mapaZamestnancu";
            var data = (new[] { new { os_cislo = (decimal?)0, jmeno = "", adresa = "", location = new double[] { 0, 0 }, kraj = "", mesto = "" } }).AsEnumerable();
            string base64 = CacheInfo.GetItemFromCache(cachekey)?.ToString();
            if (base64 == null)
            {
                string sql = "SELECT os_cislo, jmeno, prijmeni, pr_ulice, pr_cislo, psc_prech, misto_prech, prech_stat_adr, " +
                                       "tr_ulice, tr_cislo, psc_trv, misto_trv, stat_adr " +
                                       "FROM pracovnici";
                ParametryPrikazu pars = new ParametryPrikazu() { };
                DataTable zamci = SqlPrikaz.ExecuteCommand(sql, true, true, ref pars);


                double[] loc = new double[] { 0, 0 };
                string m, k;
                data = from r in zamci.AsEnumerable()
                       select (new
                       {
                           os_cislo = GetNullOrData.GetDecimal(r["os_cislo"]),
                           jmeno = GetNullOrData.GetString(r["jmeno"], "", true) + " " + GetNullOrData.GetString(r["prijmeni"], "", true),
                           adresa = getLokace(string.IsNullOrEmpty(GetNullOrData.GetString(r["misto_prech"])) ?
                                    (GetNullOrData.GetString(r["tr_ulice"], "", true) + " " + GetNullOrData.GetString(r["tr_cislo"], "", true) + ", " + GetNullOrData.GetString(r["psc_trv"], "", true) + " " + GetNullOrData.GetString(r["misto_trv"], "", true) + (string.IsNullOrEmpty(GetNullOrData.GetString(r["stat_adr"])) ? "" : (", " + GetNullOrData.GetString(r["stat_adr"], "", true)))) :
                                    (GetNullOrData.GetString(r["pr_ulice"], "", true) + " " + GetNullOrData.GetString(r["pr_cislo"], "", true) + ", " + GetNullOrData.GetString(r["psc_prech"], "", true) + " " + GetNullOrData.GetString(r["misto_prech"], "", true) + (string.IsNullOrEmpty(GetNullOrData.GetString(r["prech_stat_adr"])) ? "" : (", " + GetNullOrData.GetString(r["prech_stat_adr"], "", true)))), out loc, out k, out m),
                           location = loc,
                           kraj = k,
                           mesto = m
                       });
                CacheInfo.AddItemToCache(cachekey, Newtonsoft.Json.JsonConvert.SerializeObject(data).ZipToBase64());
            }
            else
            {
                data = Newtonsoft.Json.JsonConvert.DeserializeAnonymousType(base64.UnZipFromBase64(), data);
            }

            List<DynamicDataObject> ret = new List<DynamicDataObject>() { };
            foreach (var d in data)
            {
                DynamicDataObject o = new DynamicDataObject();
                o.Set("os_cislo", d.os_cislo);
                o.Set("jmeno", d.jmeno);
                o.Set("adresa", d.adresa);
                o.Set("location", d.location);
                o.Set("kraj", d.kraj);
                o.Set("mesto", d.mesto);
                ret.Add(o);
            }
            return ret;
        }

        private static string getLokace(string adresa, out double[] location, out string kraj, out string mesto)
        {
            string ret = "";
            kraj = ""; mesto = "";
            location = new double[] { 0, 0 };
            string apikey = "AIzaSyB46Ndy-gPSj7KWhiW0vbJm9TZkX8Kq9Ec";
            var definition = new { results = new[] { new { address_components = new[] { new { long_name = "", short_name = "", types = new[] { "" } } }, formatted_address = "", geometry = new { location = new { lat = (double)0, lng = (double)0 } } } }.ToList() };

            HttpWebRequest request = null;
            HttpWebResponse response = null;
            StreamReader reader = null;

            try
            {
                string adrkey = adresa;
                string url = string.Format("https://maps.googleapis.com/maps/api/geocode/json?address={0}&key={1}", adrkey, apikey);
                request = (HttpWebRequest)WebRequest.Create(url);
                response = (HttpWebResponse)request.GetResponse();
                reader = new StreamReader(response.GetResponseStream());
                string json = reader.ReadToEnd();
                var o = Newtonsoft.Json.JsonConvert.DeserializeAnonymousType(json, definition);

                mesto = o.results.FirstOrDefault()?.address_components.FirstOrDefault(x => x.types.Contains("administrative_area_level_2"))?.long_name;
                kraj = o.results.FirstOrDefault()?.address_components.FirstOrDefault(x => x.types.Contains("administrative_area_level_1"))?.long_name;

                location = new double[2] { o.results.FirstOrDefault().geometry.location.lat, o.results.FirstOrDefault().geometry.location.lng };
                return o.results.FirstOrDefault()?.formatted_address;
            }
            catch (Exception e)
            {

            }
            return ret;
        }

        [WebMethod]
        public static DataResult GetBulletins()
        {
            var list = KSProgram.Dotazniky.Dotaznik.GetBulletins(null, DateTime.Now, Globals.OsCislo, true, false,true);

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod]
        public static DataResult GetWarnigs()
        {
            var list = (new KSProgram.Obecne.WarningMessages()).GetAllMessages().Select(x => new { message = x }).ToList();

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod]
        public static DataResult GetOdkazy()
        {
            var list = (new KSProgram.Odkazy()).Seznam().ToList(new { nazev = "", url = "", cil = "" });

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod]
        public static DataResult GetVolnePozice()
        {
            var list = (new KSProgram.VyberovaRizeni.VyberovaRizeni()).Publikovane(true, false)
                .ToList(new { id_vybriz = 0, nazev = "" })
                .Select(x => new
                {
                    url = KSProgram.VyberovaRizeni.VyberovaRizeni.VolnaPoziceBuildURL(x.id_vybriz),
                    nazev = x.nazev
                })
                .ToList();

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod]
        public static DataResult GetNepritomnosti()
        {
            var list = (new KSProgram.Sestavy.SchvaleneNeprit()).Retrieve()
                .ToList(new { os_cislo = 0m, cislo_pom = (short)0, jmeno = "", d_1 = "", d_2 = "", d_3 = "", d_4 = "", d_5 = "", d_6 = "", d_7 = "", d_8 = "", d_9 = "", d_10 = "" });

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod(EnableSession = true)]
        public static string GetBulletinHtml(int id)
        {
            byte[] data = KSProgram.Dotazniky.Dotaznik.GetBulletins(id, null, null, null, false)?.FirstOrDefault()?.LoadHtmlByte();
            if (!PDF.IsValidPdf(data))
            {
                string html = KSProgram.Dotazniky.Dotaznik.GetBulletins(id, null, null, null, false)?.FirstOrDefault()?.LoadHtml();
                try
                {
                    byte[] pdf = Convert.FromBase64String(html);
                    if (PDF.IsValidPdf(pdf))
                    {
                        return "PDF" + html;
                    }
                    else
                    {
                        return html;
                    }
                }
                catch
                {
                    return html;
                }
            }

            //try
            //{
            //    PdfReader reader = new PdfReader(data);
            //}
            //catch
            //{
            //    string html = KSProgram.Dotazniky.Dotaznik.GetBulletins(id, null, null, null, false)?.FirstOrDefault()?.LoadHtml();
            //    try
            //    {
            //        byte[] pdf = Convert.FromBase64String(html);
            //        PdfReader reader = new PdfReader(pdf);
            //        return "PDF" + html;
            //    }
            //    catch
            //    {
            //        return html;
            //    }
                
            //}
            return "PDF"+Convert.ToBase64String(data);
        }

        [WebMethod]
        public static bool UlozDotaznikVytvoreny(string json)
        {
            //doplnit ukládání pomocí sql

            return true;
    
        }

        [WebMethod]
        public static bool UlozDotaznikOdpovedi(string json, int id, bool anonym)
        {
            KSProgram.Dotazniky.Dotaznik.SaveOdpovedi(json,id, anonym);

            return true;

        }

        [WebMethod]
        public static DataResult GetPozadavky()
        {
            PozadavekSeznam sez = new PozadavekSeznam();
            List<object> ret = (from r in (from p in sez.Retrieve(0, "vse", "vse", "mepozadavky", DateTime.MinValue, DateTime.MaxValue, DateTime.Today.AddMonths(-2).PrvniDen(), DateTime.Today.PrvniDen(), null, "vse", null).AsEnumerable()
                                           select new PozadavekSeznam.Pozad(p))
                                orderby r.datum
                                select ((object)new pozadavek()
                                {
                                    id_poz = r.id_poz,
                                    prijmeni = r.prijmeni,
                                    jmeno = r.jmeno,
                                    typ = r.typ,
                                    text = r.text,
                                    stav = r.stav
                                })).ToList();

            return new DataResult
            {
                Data = ret,
                Count = ret.Count
            };
        }

        [WebMethod]
        public static string GetOtazky(int id)
        {
            KSProgram.Dotazniky.Dotaznik dotaznik = KSProgram.Dotazniky.Dotaznik.GetBulletins(id, null, null, null).FirstOrDefault();
            string html = dotaznik.LoadHtml();
            return html;
        }

        public class DataResult
        {
            public IEnumerable<object> Data { get; set; }
            public int Count { get; set; }
        }

        public class PozadavkyDataResult
        {
            public List<object> Data { get; set; }
            public int Count { get; set; }
        }
        public class pozadavek
        {
            public int? id_poz { get; set; }
            public string jmeno { get; set; }
            public string prijmeni { get; set; }
            public string typ { get; set; }
            public string text { get; set; }
            public string stav { get; set; }
        }
    }
}