﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;
using System.Threading.Tasks;
using KSProgram.Diagnostics;
using KSProgram;

namespace KSPortal
{
    /// <summary>
    /// Logování přihlašování
    /// </summary>
    public class LogonLogger
    {
        /// <summary>
        /// Zápis do logu
        /// </summary>
        /// <param name="login">Přihlašovací jméno</param>
        /// <param name="result">Výsledek pokusu o přihlášení</param>
        public static void WriteLine(string login, string result)
        {
            //kam budu zapisovat
            string rootDir = AppDomain.CurrentDomain.BaseDirectory;
            string logsDir = rootDir + @"Logs\";
            string logFile = logsDir + @"Logons.log";
            if (!Directory.Exists(logsDir))
                Directory.CreateDirectory(logsDir); //na toto určitě nemá portál oprávnění, ale dobrá, nechám to tu
            if (!File.Exists(logFile))
            {
                //PR 11.05.2022 try catch
                try
                {
                    FileStream fs = File.Create(logFile);
                    fs.Close();
                }
                catch (UnauthorizedAccessException exc)
                {
                    throw new UnauthorizedAccessException("Do adresáře Logs nelze zapisovat! Přidělte skupině Autheticated Users právo měnit obsah složek Logs a Temp!", exc);
                }
            }

            //data
            Dictionary<string, string> myDict = new Dictionary<string,string>();
            myDict.Add("Time", DateTime.Now.ToString("s"));
            myDict.Add("HostIp", HttpContext.Current.Request.UserHostAddress);
            myDict.Add("Login", login);
            string typOvereni = KSProgram.Funkce.AppConfiguration.GetAppSetting<string>("TypOvereni", "kspu");
            myDict.Add("AuthenticationMethod", typOvereni);
            myDict.Add("Result", result);
            if (HttpContext.Current.Session != null)
                myDict.Add("SessionId", HttpContext.Current.Session.SessionID);

            //zápis do logu
            string line = string.Join(";", myDict.Select(x => x.Key + "=" + x.Value).ToArray());

            //StreamWriter sw = File.AppendText(logFile);
            //sw.AutoFlush = true;
            //sw.WriteLine(line);
            //sw.Close();
            //sw.Dispose();

            //PR 19.10.2020 vícenásobný přístup k souboru (současně se hlásící uživatelé)            
            int maxPokusu = 10;
            for (int pokus = 1; pokus <= maxPokusu; pokus++)
            {
                try
                {
                    InternalWriteLine(logFile, line);
                    break;
                }
                catch (UnauthorizedAccessException exc) //PR 11.05.2022 při problémech s právy zobrazím chybu ihned
                {
                    throw new UnauthorizedAccessException("Do adresáře Logs nelze zapisovat! Přidělte skupině Autheticated Users právo měnit obsah složek Logs a Temp!", exc);
                }
                catch (IOException e1) when (pokus < maxPokusu)//Soubor je využíván jiným procesem, disk je plný apod.
                {
                    //Při 10. pokusu díky podmínce "when" již ke spolknutí IOException nedojde a vybublá (pokud by to byla chyba typu disk full, tak je dobře, že se zobrazí)
                    KSTrace.WriteLine($"Systému se nepodařilo zapsat do souboru Logons.log, pokus č. {pokus} , " + e1.Message);
                    Task.Delay(100).Wait(); //System.Threading.Thread.Sleep(100);
                }
            }
            
        }

        private static void InternalWriteLine(string logFile, string line)
        {
            StreamWriter sw = File.AppendText(logFile);
            sw.AutoFlush = true;
            sw.WriteLine(line);
            sw.Close();
            sw.Dispose();
        }
        
    }
}