﻿using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using KSProgram.Data;
using KSProgram.WebControls;
using KSProgram;
using KSProgram.Prava;
using KSProgram.Funkce;
using KSProgram.Obecne;
using System.Globalization;
using System.Threading;
using System.ComponentModel;
using KSProgram.Security;
using KSProgram.Extensions;
using Telerik.Web.UI;
using Telerik.Web.Device.Detection;
using Newtonsoft.Json.Linq;

/// <summary>
/// Předek stránek. Nevyžaduje MasterPage.
/// </summary>
public class TPrePage : System.Web.UI.Page
{
    #region Enums
    /// <summary>
    /// Témata stránek
    /// </summary>
    public enum UserThemes
    {
        None
    }
    #endregion

    #region Variables
    private Boolean _defaultDatabase = false;
    private UserThemes _userTheme = UserThemes.None;
    private Boolean _isLoginPage = false;
    #endregion

    #region Properties
    /// <summary>
    /// Zda-li se má vytvořit připojení k default databázi, pokud není db připojení nastaveno.
    /// Použití: pro public stránky, které nemají výběr databáze nebo si nepřebírají DB z url.
    /// </summary>
    [DefaultValue(false)]
    protected Boolean DefaultDatabase
    {
        set { _defaultDatabase = value; }
        get { return _defaultDatabase; }
    }

    /// <summary>
    /// Grafické téma stránky
    /// </summary>
    [Obsolete("Tato vlastnost se už nepoužívá. MD 5.8.2014", true)]
    [DefaultValue(TPage.UserThemes.None)]
    protected UserThemes UserTheme
    {
        set { _userTheme = value; }
        get { return _userTheme; }
    }
    /// <summary>
    /// Zda-li stránka slouží k přihlašování
    /// </summary>
    [DefaultValue(false)]
    protected Boolean IsLoginPage
    {
        set { _isLoginPage = value; }
        get { return _isLoginPage; }
    }
    /// <summary>
    /// Pokud jde o známé zařízení, vrací rozměry. Pro desktop je rozměr 0
    /// https://docs.telerik.com/devtools/aspnet-ajax/shared-utilities/raddevicedetectionframework
    /// </summary>
    public DeviceScreenDimensions ZarizeniRozmery
    {
        get
        {
            return Detector.GetScreenDimensions(Request.UserAgent);
        }
    }
    /// <summary>
    /// Pokud jde o známé zařízení, vrací jeho velikost.
    /// https://docs.telerik.com/devtools/aspnet-ajax/shared-utilities/raddevicedetectionframework
    /// </summary>
    public DeviceScreenSize ZarizeniVelikost
    {
        get
        {
            return Detector.GetScreenSize(Request.UserAgent);
        }
    }

    /// <summary>
    /// Grafické téma stránky
    /// </summary>
    public override string StyleSheetTheme
    {
        get
        {
            //MiK 3.3.2014 toto odskinuje portál (Standartní kontrolky)
            //return string.Empty; 

            //MD 3.12.3013 Pokud je definován Telerik.Skin, použije se
            string telerikSkin = AppConfiguration.GetAppSetting<string>("Telerik.Skin", null);
            string baseSkin = base.StyleSheetTheme;
            if (string.IsNullOrEmpty(baseSkin)) return telerikSkin;

            return base.StyleSheetTheme;
        }
    }
    /// <summary>
    /// Zarovnání obsahu stránky
    /// </summary>
    [DefaultValue(HorizontalAlign.Center)]
    protected virtual HorizontalAlign ContentHorizontalAlign
    {
        set
        {
            ViewState["ContentHorizontalAlign"] = value;
        }
        get
        {
            object obj = ViewState["ContentHorizontalAlign"];
            return (obj == null) ? HorizontalAlign.Center : (HorizontalAlign)obj;
        }
    }
    /// <summary>
    /// 
    /// </summary>
    protected virtual String ContentCssClass
    {
        set
        {
            ViewState["ContentCssClass"] = value;
        }
        get
        {
            object obj = ViewState["ContentCssClass"];
            return (obj == null) ? "" : (String)obj;
        }
    }
    /// <summary>
    /// Zda je na stránce umožněno přepínání jazyků
    /// </summary>
    [DefaultValue(true)]
    protected bool EnableChangeLanguage
    {
        set
        {
            ViewState["chlang"] = value;
        }
        get
        {
            return (ViewState["chlang"] == null) ? true : (bool)ViewState["chlang"];
        }
    }

    /// <summary>
    /// Zda je na stránce dostupné Progress Kendo UI for jQuerry
    /// </summary>
    [DefaultValue(false)]
    protected bool EnableKendo
    {
        set
        {
            ViewState["enableKendo"] = value;
        }
        get
        {
            return (ViewState["enableKendo"] == null) ? false : (bool)ViewState["enableKendo"];
        }
    }

    /// <summary>
    /// Dočasně než se vyřeší problém při přeházení sloupců v seznamu zaměstnanců
    /// </summary>
    public override bool EnableEventValidation
    {
        get
        {
            return false;
        }
        set
        {
            base.EnableEventValidation = false;
        }
    }
    #endregion

    /// <summary>
    /// Jméno PostBack event položky vyvolávající postback.
    /// </summary>
    public const string PostBackEventTarget = "__EVENTTARGET";

    public TPrePage()
    {
        this.PreInit += new EventHandler(Page_PreInit);
        this.Init += new EventHandler(Page_Init);
        this.InitComplete += new EventHandler(Page_InitComplete);
        this.PreLoad += new EventHandler(Page_PreLoad);
        this.Load += new EventHandler(this.Page_Load);
        this.LoadComplete += new EventHandler(this.Page_LoadComplete);
        this.PreRender += new EventHandler(this.Page_PreRender);
        this.PreRenderComplete += new EventHandler(this.Page_PreRenderComplete);
        //savestate?
        this.SaveStateComplete += new EventHandler(this.Page_SaveStateComplete);
    }

    protected override void InitializeCulture()
    {
        ///<remarks>
        ///Zkontroluje jestli nastal postback, nelze použít v této metodě IsPostBack
        ///když tato vlastnost není nastavena
        ///</remarks>
        //if ((Request[PostBackEventTarget] != null)&&(0==1))
        if (Request[PostBackEventTarget] != null)
        {
            string controlID = Request[PostBackEventTarget];
            string CsControlName = "ctl00_language_ImageButton_cs";
            string EnControlName = "ctl00_language_ImageButton_en";
            string DeControlName = "ctl00_language_ImageButton_de";
            string SkControlName = "ctl00_language_ImageButton_sk";
            if (controlID.Equals(CsControlName))
                SetCulture("cs-CZ", true);
            else if (controlID.Equals(EnControlName))
            {
                SetCulture("en-US", true);
            }
            else if (controlID.Equals(DeControlName))
                SetCulture("de-DE", true);
            else if (controlID.Equals(SkControlName))
                SetCulture("sk-SK", true);

        }
        // MiK 1.7.2014 - zapoznámkováno a přemístěno do Global.asax - Application_BeginRequest
        // tam se řeší nastavení CurrentUICulture i v asynchronních požadavcích webových služeb
        //if (Request.Cookies["Culture"] != null)
        //{
        //    string lang = Request.Cookies["Culture"].Value;
        //    //if ((lang == "cs-CZ") || (lang == "en-US"))
        //    //PH 11.5.2011
        //    if ((lang == "cs-CZ") || (lang == "en-US") || (lang == "de-DE"))
        //    {
        //        // Thread.CurrentThread.CurrentUICulture = new CultureInfo(lang);PH 25.2.2013 zapoznámkováno bude se nastavovat ve web.config
        //        // Thread.CurrentThread.CurrentCulture = Thread.CurrentThread.CurrentUICulture;
        //        Thread.CurrentThread.CurrentUICulture = new CultureInfo(lang);
        //    }
        //    else if (lang == "sk-SK")
        //    {
        //        //Thread.CurrentThread.CurrentUICulture = new CultureInfo(lang);PH 25.2.2013 zapoznámkováno bude se nastavovat ve web.config
        //        Thread.CurrentThread.CurrentUICulture = new CultureInfo("sk-SK"); ;
        //    }
        //}
        base.InitializeCulture();
        //PR 19.09.2018 pokud formát datumu začíná měsícem (americký formát), tak portál nemůže fungovat správně, protože jsou skoro všude natvrdo nastaveny formáty d.M.yyyy.
        //Pak to končí chybou při konverzi FormatException "String was not recognized as a valid DateTime" (ODS.HandleInsert->....Convert.ToDateTime->DateTimeParse.Parse)
        //if (CultureInfo.CurrentCulture.DateTimeFormat.ShortDatePattern.StartsWith("M"))
        //    throw new Exception("Váš webový server používá americký formát datumů "+ CultureInfo.CurrentCulture.DateTimeFormat.ShortDatePattern + "! Aby portál fungoval správně, je třeba nastavit v IIS/ksportal/.NET Globalization položky Culture a UICulture na cs-CZ!");
        //PR 08.10.2018 zase jsem to zrušil, při změně jazyka na en-US, se stejně formát zase změní na M/d/yyyy. U RadDatePickeru to nevadí, protože vlastnost SelectedDate je typu dateTime a 
        //konverze tedy probíhá uvnitř kontrolky s nastaveným formátem. Problém je u TextBox, HiddenField, když se při nastavení hodnoty použije formát d.M.yyyy.
    }

    protected virtual void Page_PreInit(object sender, System.EventArgs e)
    {
        //PR 15.04.2021 
        if (DefaultDatabase)
        {
            if (!Globals.IsDbConected)
                Globals.Db = DatabasePodleParam.Create(null);
        }
    }

    protected override void OnPreInit(EventArgs e)
    {
        string wpfjsonss = SessionInfo.GetVariable("WebPlatformJson")?.ToString();
        if (!string.IsNullOrEmpty(wpfjsonss))
        {
            Globals.WebPlatform = JObject.Parse(wpfjsonss);
        }
        base.OnPreInit(e);
    }

    protected override void OnInit(EventArgs e) //PH 5.1.2013 kvůli bezpečnosti
    {
        this.RegisterRequiresViewStateEncryption();
        base.OnInit(e);
    }

    protected virtual void Page_Init(object sender, System.EventArgs e)
    {
        //MD 10.4.2012 reference na JQuery
        //-----------------------------------------------------------
        HtmlLink jqcss = new HtmlLink();
        jqcss.Href = ResolveUrl("~/css/jquery-ui.css");
        jqcss.Attributes["rel"] = "stylesheet";
        jqcss.Attributes["type"] = "text/css";
        HtmlLink jqcss2 = new HtmlLink();
        jqcss2.Href = ResolveUrl("~/css/jquery-ui.structure.css");
        jqcss2.Attributes["rel"] = "stylesheet";
        jqcss2.Attributes["type"] = "text/css";
        HtmlLink jqcss3 = new HtmlLink();
        jqcss3.Href = ResolveUrl("~/css/jquery-ui.theme.css");
        jqcss3.Attributes["rel"] = "stylesheet";
        jqcss3.Attributes["type"] = "text/css";
        Page.Header.Controls.AddAt(0, jqcss);
        //if (!Page.ClientScript.IsClientScriptIncludeRegistered("jquery-3.0.0.min.js"))
        //    Page.ClientScript.RegisterClientScriptInclude("jquery-3.6.0.min.js", ResolveUrl("~/Scripts/jquery-3.6.0.min.js"));
        //if (!Page.ClientScript.IsClientScriptIncludeRegistered("jquery-ui.js"))
        //    Page.ClientScript.RegisterClientScriptInclude("jquery-ui.js", ResolveUrl("~/Scripts/jquery-ui.js"));
        //if (!Page.ClientScript.IsClientScriptIncludeRegistered("jquery.helper.js"))
        //    Page.ClientScript.RegisterClientScriptInclude("jquery.helper.js", ResolveUrl("~/Scripts/jquery.helper.js"));
        TRegisteredScripts.AddScriptToPage(this.Page, "~/Scripts/jquery-3.6.1.min.js", "jquery");
        TRegisteredScripts.AddScriptToPage(this.Page, "~/Scripts/jquery-ui.js", "jquery-ui");
        TRegisteredScripts.AddScriptToPage(this.Page, "~/Scripts/jquery.helper.js", "jquery.helper");
        if (EnableKendo)
        {
            TRegisteredScripts.AddCssStyleToPage(this.Page, "~/kendo/styles/kendo.common.min.css");
            TRegisteredScripts.AddCssStyleToPage(this.Page, "~/Skins/kendoMetro/kendoMetro.css");
            
            TRegisteredScripts.AddScriptToPage(this.Page, "~/kendo/js/kendo.all.min.js", "kendo.all");
        }
        //-----------------------------------------------------------
    }

    protected virtual void Page_InitComplete(object sender, System.EventArgs e)
    {
    }

    protected virtual void Page_PreLoad(object sender, System.EventArgs e)
    {
    }
    //protected override void OnError(EventArgs e)
    //{
    //    HttpContext context = HttpContext.Current;

    //    Exception exc = null;
    //    exc = context.Server.GetLastError();

    //    if (exc is System.Web.HttpUnhandledException && exc.InnerException != null)
    //    {
    //        exc = exc.InnerException;
    //    }

    //    if (exc != null)
    //    {
    //        Exception baseExc = exc.GetBaseException();
    //        string baseExcType = baseExc.GetType().Name;
    //        bool isOznameni = baseExcType == "NoticeException" ? true : false;

    //        if (isOznameni)
    //        {
    //            MessageBox.Alert(this.Page, exc);
    //            context.Server.ClearError();
    //            return;
    //        }
    //    }

    //    base.OnError(e);
    //}
    protected virtual void Page_Load(object sender, System.EventArgs e)
    {
        //PH 3.1.2013 ochrana proti SQLI
        if (InputChecker.Enabled)
        {
            /*
            if (Request.RequestType == "POST")
            {
                for (int i = 0; i < Request.Form.Count; i++)
                {
                    if ((Request.Form.Keys[i] != "__LASTFOCUS") &&
                    (Request.Form.Keys[i] != "__EVENTTARGET") &&
                    (Request.Form.Keys[i] != "__EVENTARGUMENT") &&
                    (Request.Form.Keys[i] != "__VIEWSTATE") &&
                    (Request.Form.Keys[i] != "__VIEWSTATEENCRYPTED") &&
                    (Request.Form.Keys[i] != "__PREVIOUSPAGE") &&
                    (Request.Form.Keys[i] != "__EVENTVALIDATION") &&
                    (Request.Form.Keys[i] != "ctl00_RadScriptManager1_TSM"))
                    {
                        //if Request.Form.Keys[i] != "__VIEWSTATE"
                        //{

                        //    Request.Form[i]
                        //    continue;
                        //}
                        string val = Request.Form[Request.Form.Keys[i]];
                        if ((val == "<") || (val == ">") || (val == "<=") || (val == ">=") || (val == "=")) continue; //PH 5.1.2014 pokud je to tam samostatně tak to nezpůsobí bezp. problém
                        try
                        {
                            val = InputChecker.Text(Request.Form[Request.Form.Keys[i]], Request.Form.Keys[i]);
                        }
                        catch (SecurityException ex)
                        {
                            throw;
                        }
                    }
                }
            }
            * */

            if ((!string.IsNullOrEmpty(Request.Params["__VIEWSTATE"]) && string.IsNullOrEmpty(Request.Form["__VIEWSTATE"])) ||
            (!string.IsNullOrEmpty(Request.Params["__EVENTTARGET"]) && string.IsNullOrEmpty(Request.Form["__EVENTTARGET"])) ||
            (!string.IsNullOrEmpty(Request.Params["__EVENTARGUMENT"]) && string.IsNullOrEmpty(Request.Form["__EVENTARGUMENT"])) ||
            (!string.IsNullOrEmpty(Request.Params["__LASTFOCUS"]) && string.IsNullOrEmpty(Request.Form["__LASTFOCUS"])) ||
            (!string.IsNullOrEmpty(Request.Params["__VIEWSTATEENCRYPTED"]) && string.IsNullOrEmpty(Request.Form["__VIEWSTATEENCRYPTED"])))
            {
                throw new Exception("Viewstate, EventTarget, EventArgument, LastFocus, ViewStateEncrytped nemůže být ve formuláři.");
            }
            ////PR 22.11.2021 nastavení ValidateRequestMode na disabled nezabíralo, zde se musí přistupovat přes Request.Unvalidated
            //if ((!string.IsNullOrEmpty(Request.Unvalidated.Params["__VIEWSTATE"]) && string.IsNullOrEmpty(Request.Unvalidated.Form["__VIEWSTATE"])) ||
            //(!string.IsNullOrEmpty(Request.Unvalidated.Params["__EVENTTARGET"]) && string.IsNullOrEmpty(Request.Unvalidated.Form["__EVENTTARGET"])) ||
            //(!string.IsNullOrEmpty(Request.Unvalidated.Params["__EVENTARGUMENT"]) && string.IsNullOrEmpty(Request.Unvalidated.Form["__EVENTARGUMENT"])) ||
            //(!string.IsNullOrEmpty(Request.Unvalidated.Params["__LASTFOCUS"]) && string.IsNullOrEmpty(Request.Unvalidated.Form["__LASTFOCUS"])) ||
            //(!string.IsNullOrEmpty(Request.Unvalidated.Params["__VIEWSTATEENCRYPTED"]) && string.IsNullOrEmpty(Request.Unvalidated.Form["__VIEWSTATEENCRYPTED"])))
            //{
            //    throw new Exception("Viewstate, EventTarget, EventArgument, LastFocus, ViewStateEncrytped nemůže být ve formuláři.");
            //}
        }


        string pom = AppConfiguration.GetAppSetting<string>("Cache", string.Empty);
        if (pom.ToUpper() == "NOPAGE")
        {
            Response.Cache.SetCacheability(HttpCacheability.NoCache); //PH 5.8.2008
            Response.Cache.SetAllowResponseInBrowserHistory(false);
            Response.Cache.SetNoStore();
            Response.Cache.SetNoServerCaching();
        }
        // MIK 10.10.2013 Nová Masterpage
        if (!IsPostBack)
        {
            if (Master != null)
            {
                object log_pan = Master.FindControl("Panel_loginfo");
                if (log_pan != null)
                    ((Control)log_pan).Visible = IsLoginPage;


                //skrývá vlaječky pro přepínání jazyka, na některých stránkách by to nemělo být vidět
                if (!EnableChangeLanguage)
                {
                    Master.FindControl("P_flags_m").Visible = false;
                    //Master.FindControl("flag_cs").Visible = false;
                    //Master.FindControl("flag_en").Visible = false;
                    //Master.FindControl("flag_de").Visible = false;
                    //Master.FindControl("flag_sk").Visible = false;
                }
                //PH 17.12.2012 dosud nemáme naplněn slovník pro SK 
                //PH 24.3.2015 zapoznámkováno slovnik sk je doplněn, alespoň pro tuto chvíli
                //Master.FindControl("flag_sk").Visible = false;

            }
        }
        string wpfjsonss = SessionInfo.GetVariable("WebPlatformJson")?.ToString();
        HiddenField HF_platform_json = (HiddenField)Master?.FindControl("HF_platform_json");
        if (HF_platform_json != null && !string.IsNullOrEmpty(HF_platform_json.Value))
        {
            SessionInfo.SetVariable("WebPlatformJson", HF_platform_json.Value);
            Globals.WebPlatform = JObject.Parse(HF_platform_json.Value);
        }
        else if (!string.IsNullOrEmpty(wpfjsonss))
        {
            Globals.WebPlatform = JObject.Parse(wpfjsonss);
        }
    }

    protected virtual void Page_LoadComplete(object sender, System.EventArgs e)
    { }

    protected virtual void Page_PreRender(object sender, System.EventArgs e)
    {
        //PR 18.10.2012 přesunuto do Page_PreRenderComplete
        /*if (!IsPostBack)
        {
            NastavPrava();
        }*/
    }

    protected virtual void Page_PreRenderComplete(object sender, System.EventArgs e)
    {
        //provádím lokalizaci položek
        WebControlHelper.ProjdiControls(this, string.Empty, WebControlHelper.ZmenaTextu);

        //MD 25.4.2014 přepnutí do nejvyššího módu IE
        //MD 14.3.2016 pozor!, toto nastavení překrývá nastavení zpětné kompatibility pro intranet (v nastavení pod F12).
        //  Pak se zdá, že IE není v režimu zpětné kompatibility, i když stále je!
        HtmlMeta modeMeta = new HtmlMeta();
        modeMeta.HttpEquiv = "X-UA-Compatible";
        modeMeta.Content = "IE=Edge";
        this.Page.Header.Controls.AddAt(0, modeMeta);

        //PR 18.10.2012 přesunuto z Page_PreRender 
        //if (!IsPostBack)
        {
            NastavPrava();
        }

        //MiKo: 23.9.2021 Nastavím TemporaryFolder všech RadAsyncUpload na adresář ~/Temp
        this.FindControlRecursive<RadAsyncUpload>()
            .ForEach(x => x.TemporaryFolder = "~/Temp");
    }



    protected virtual void Page_SaveStateComplete(object sender, System.EventArgs e)
    { }

    /// <summary>
    /// Rekurzivně vyhledává childcontrol v hierarchii vnořených controlů.
    /// </summary>
    /// <param name="id">ID hledaného controlu <see cref="string"/></param>
    /// <returns>Nalezený childcontrol <see cref="Control"/>. Pokud ho nenajde, vrací null. </returns>
    public virtual Control FindControlRecursive(string id)
    {
        Control ctrl = FindControl(id);
        if (ctrl == null)
        {
            if (Controls.Count >= 0)
            {
                return FindControlRecursive(id, Controls);
            }
        }
        else
        {
            //byl nalezen
            return ctrl;
        }
        return null;
    }


    /// <summary>
    /// Aplikuje práva pro celou stránku
    /// </summary>
    public virtual void NastavPrava()
    {
        foreach (Control ctrl in Controls)
        {
            NastavPrava(ctrl);
        }
    }
    /// <summary>
    /// Aplikuje práva pro danou kontrolku
    /// </summary>
    /// <param name="ctrl"></param>
    protected virtual void NastavPrava(Control ctrl)
    {
        int id_prava = 0;
        int id_prava_proh = 0;
        bool povolen = false;
        if (ctrl.GetType() == typeof(TLinkButton))
        {
            id_prava = ((TLinkButton)ctrl).IdPrava;
        }
        else if (ctrl.GetType() == typeof(TButton))
        {
            id_prava = ((TButton)ctrl).IdPrava;
        }
        else if (ctrl.GetType() == typeof(TRadButton))
        {
            id_prava = ((TRadButton)ctrl).IdPrava;
        }
        else if (ctrl.GetType() == typeof(TGridView) || ctrl.GetType() == typeof(TGridViewBulk))
        {
            id_prava = ((TGridView)ctrl).IdPravaEditace;
            id_prava_proh = ((TGridView)ctrl).IdPravaProhlizeni;
        }
        else if (ctrl.GetType() == typeof(TRadGrid)) //MiK 14.10.2014 přidána větev pro RadGrid
        {
            id_prava = ((TRadGrid)ctrl).IdPravaEditace;
        }
        else if (ctrl.GetType() == typeof(TTextBox))
        {
            id_prava = ((TTextBox)ctrl).IdPrava;
        }
        else if (ctrl.GetType() == typeof(TImageButton))//MiK 7.11.2012 přidána větev pro TImageButton
        {
            id_prava = ((TImageButton)ctrl).IdPrava;
        }
        else if (ctrl.GetType() == typeof(TFormView))
        {
            id_prava = ((TFormView)ctrl).IdPravaEditace;
            id_prava_proh = ((TFormView)ctrl).IdPravaProhlizeni;
        }
        else if( ctrl.GetType() == typeof(TRadToggleButton) )
        {
            id_prava = ((TRadToggleButton)ctrl).IdPrava;
        }
        else if( ctrl.GetType() == typeof(TRadImageButton) )
        {
            id_prava = ((TRadImageButton)ctrl).IdPrava;
        }
        else if( ctrl.GetType() == typeof(TRadCheckBox) )
        {
            id_prava = ((TRadCheckBox)ctrl).IdPrava;
        }
        else if( ctrl.GetType() == typeof(TRadLinkButton) )
        {
            id_prava = ((TRadLinkButton)ctrl).IdPrava;
        }
        else if (ctrl.GetType() == typeof(TRadPushButton))
        {
            id_prava = ((TRadPushButton)ctrl).IdPrava;
        }
 

        if (id_prava != 0)
        {
            povolen = PravaAkce.PristupPovolen(id_prava);

            if (ctrl.GetType() == typeof(TGridView) || ctrl.GetType() == typeof(TGridViewBulk))
            {
                ((TGridView)ctrl).JePravoNaAkciEditace = povolen;
            }
            else if (ctrl.GetType() == typeof(TRadGrid))
            {
                ((TRadGrid)ctrl).EditaceZakazana = !povolen;
            }
            else if (ctrl.GetType() == typeof(TFormView))
            {
                ((TFormView)ctrl).JePravoNaAkciEditace = povolen;
            }
            else
            {
                //((WebControl)ctrl).Enabled = povolen;
                ////((WebControl)ctrl).ToolTip = (povolen ? "" : " (přístup zakázán)");//PR 18.10.2012 podmínka byla opačně  //MK 7.11.2012 tento kód vždy přepsal ToolTip
                //if (!povolen)
                //{
                //    ((WebControl)ctrl).ToolTip = "(přístup zakázán)";
                //}
                if (!povolen) //MiKo Funkcionalita přepsána tak, aby se controlka pouze zakazovala
                {
                    ((WebControl)ctrl).Enabled = false;
                    ((WebControl)ctrl).ToolTip = WebControlHelper.ZmenaTextu("[%(přístup zakázán)%]");
                }
            }
        }
        if (id_prava_proh != 0)
        {
            if (ctrl.GetType() == typeof(TGridView) || ctrl.GetType() == typeof(TGridViewBulk))
            {
                ((TGridView)ctrl).ProhlizeniZakazano = !PravaAkce.PristupPovolen(id_prava_proh);
            }
            else if (ctrl.GetType() == typeof(TFormView))
            {
                ((TFormView)ctrl).ProhlizeniZakazano = !PravaAkce.PristupPovolen(id_prava_proh);
            }
        }

        //zjistím jestli má potomky a pokud ano tak je také prověřuju
        //-----------------------------------------------------
        if (ctrl.HasControls())
        {
            foreach (Control c in ctrl.Controls)
            {
                NastavPrava(c);
            }
        }
    }


    /// <summary>
    /// Rekurzivně vyhledává childcontrol v hierarchii vnořených controlů.
    /// </summary>
    /// <param name="id">ID hledaného controlu <see cref="string"/></param>
    /// <param name="controls">Prohledáváná kolekce controlů <see cref="ControlCollection"/></param>
    /// <returns>Nalezený childcontrol <see cref="Control"/>. Pokud ho nenajde, vrací null. </returns>
    protected virtual Control FindControlRecursive(string id, ControlCollection controls)
    {
        foreach (Control control in controls)
        {
            if (control.ID == id) return control;
            if (control.HasControls())
            {
                Control childControl = FindControlRecursive(id, control.Controls);
                if (childControl != null) return childControl;
            }
        }
        return null;
    }
    /// <summary>
    /// Rekurzivně vyhledává childControl v zadaném controlu 
    /// </summary>
    /// <param name="id">ID hledaného controlu</param>
    /// <param name="control">Výchozí control, ve kterém se hledá.</param>
    /// <returns>Nalezený control. Pokud není nalezen, vrací se null.</returns>
    protected virtual Control FindControlRecursive(string id, Control control)
    {
        Control retControl = control.FindControl(id);
        if (retControl != null) return retControl;

        if (control.HasControls())
        {
            foreach (Control child in control.Controls)
            {
                if (child != null)
                {
                    retControl = FindControlRecursive(id, child);
                    if (retControl != null) return retControl;
                }
            }
        }
        return null;
    }
    /// <summary>
    /// Nalezne nejbližší nadřazený Control zadaného typu
    /// </summary>
    /// <param name="C">Control ke kterému se hledá nadřazený</param>
    /// <param name="T">Typ nadřazeného Control</param>
    /// <returns></returns>
    protected virtual Control FindParentControl(Control C, Type T)
    {
        if (C.Parent == null)
            return null;
        else if (C.Parent.GetType() == T)
            return C.Parent;
        else return FindParentControl(C.Parent, T);
    }
    /// <summary>
    /// Nalezne nejbližší nadřazený Control zadaného typu a ID
    /// </summary>
    /// <param name="C">Control ke kterému se hledá nadřazený</param>
    /// <param name="T">Typ nadřazeného Control</param>
    /// <param name="ID">ID nadřazeného Control</param>
    /// <returns></returns>
    protected virtual Control FindParentControl(Control C, Type T, String ID)
    {
        if (C.Parent == null)
            return null;
        else if (C.Parent.GetType() == T && C.Parent.ID == ID)
            return C.Parent;
        else return FindParentControl(C.Parent, T, ID);
    }
    /// <summary>
    /// Nalezne N-tý nadřazený Control zadaného typu. (Např. druhé nadřazené GridView)
    /// </summary>
    /// <param name="C">Control ke kterému se hledá nadřazený</param>
    /// <param name="T">Typ nadřazeného Control</param>
    /// <param name="N">pořadí hledaného Control (1 až x)</param>
    /// <returns></returns>
    protected virtual Control FindParentControl(Control C, Type T, Int16 N)
    {
        if (C.Parent == null)
            return null;
        else if (C.Parent.GetType() == T)
        {
            N -= 1;
            if (N == 0)
                return C.Parent;
            else
                return FindParentControl(C.Parent, T, N);
        }
        else return FindParentControl(C.Parent, T, N);
    }

    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        //MD 6.11.2013 zarovnání obsahu MasterPage
        //-----------------------------------------
        if (Master != null)
        {
            // odstraněno s novou master
            //HtmlTable table = Master.FindControl("TableContent") as HtmlTable;
            //if (table != null)
            //{
            //    if (ContentHorizontalAlign == HorizontalAlign.Center)
            //    {
            //        //zarovnání obsahu na střed
            //        table.Attributes.Add("class", "hcenter");
            //    }
            //}
            // náhrada
            HtmlTableCell table_main_content_td = Master.FindControl("table_main_content_td") as HtmlTableCell;
            if (table_main_content_td != null)
            {
                string cssclass = table_main_content_td.Attributes["class"];
                cssclass = string.IsNullOrEmpty(cssclass) ? "" : cssclass;

                if (!string.IsNullOrEmpty(ContentCssClass))
                {
                    cssclass = cssclass.Contains(ContentCssClass) ? cssclass : (cssclass + (string.IsNullOrEmpty(cssclass) ? ContentCssClass : (" " + ContentCssClass)));
                }
                // v nové master nemá smysl 
                //if (ContentHorizontalAlign == HorizontalAlign.Center)
                //{
                //    //zarovnání obsahu na střed
                //    cssclass = cssclass.Contains("hcenter") ? cssclass : (cssclass + (string.IsNullOrEmpty(cssclass) ? "hcenter" : " hcenter"));
                //}
                table_main_content_td.Attributes.Add("class", cssclass);
            }
        }
        //-----------------------------------------
        if (!IsPostBack)
        {
            foreach (RadWindow w in this.FindControlRecursive<RadWindow>())
            {
                w.Localization.Cancel = GetGlobalResourceObject("RadWindow", "Cancel").ToString();
                w.Localization.Close = GetGlobalResourceObject("RadWindow", "Close").ToString();
                w.Localization.Maximize = GetGlobalResourceObject("RadWindow", "Maximize").ToString();
                w.Localization.Minimize = GetGlobalResourceObject("RadWindow", "Minimize").ToString();
                w.Localization.No = GetGlobalResourceObject("RadWindow", "No").ToString();
                w.Localization.OK = GetGlobalResourceObject("RadWindow", "OK").ToString();
                w.Localization.PinOff = GetGlobalResourceObject("RadWindow", "PinOff").ToString();
                w.Localization.PinOn = GetGlobalResourceObject("RadWindow", "PinOn").ToString();
                w.Localization.Reload = GetGlobalResourceObject("RadWindow", "Reload").ToString();
                w.Localization.Restore = GetGlobalResourceObject("RadWindow", "Restore").ToString();
                w.Localization.Yes = GetGlobalResourceObject("RadWindow", "Yes").ToString();
            }
        }
    }

    /// <Summary>
    /// Sets the current UICulture and CurrentCulture based on
    /// the arguments
    /// </Summary>
    /// <PARAM name="name"></PARAM>
    /// <PARAM name="locale"></PARAM>
    protected void SetCulture(string name, bool redir)
    {
        Thread.CurrentThread.CurrentUICulture = new CultureInfo(name);
        Thread.CurrentThread.CurrentCulture = Thread.CurrentThread.CurrentUICulture;
        Thread.CurrentThread.CurrentCulture.DateTimeFormat.CalendarWeekRule = CalendarWeekRule.FirstFourDayWeek;

        //uložím info do cookiny
        //HttpCookie myCookie = new HttpCookie("Culture");
        //myCookie.Value = name;
        //myCookie.Expires = DateTime.Now.AddDays(30d);
        //Response.Cookies.Add(myCookie);
        //MiK 8.1.2014
        F_Cookie.Add("Culture", name, DateTime.Now.AddDays(30d));

        ///<remarks>
        ///Saving the current thread's culture set by the User in the Session
        ///so that it can be used across the pages in the current application.
        ///</remarks>
        //SessionInfo.SetVariable("Culture",Thread.CurrentThread.CurrentUICulture);
        if (redir)
            Response.Redirect(Request.RawUrl);


    }
}
