﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using KSPortal.WebServices;
using KSProgram;
using KSProgram.Exceptions;

namespace KSPortal.Public
{
    public partial class TokenLogin : TPrePage
    {
        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);

            if (!IsPostBack)
            {
                //Zda povolit registraci uživatelů
                string registrace = KSProgram.Funkce.AppConfiguration.GetAppSetting<string>("Registrace", null);
                LB_Registrace.Visible = (registrace == "user" | registrace == "email");
            }

            string token = Request.QueryString["token"];
            if (string.IsNullOrWhiteSpace(token))
            {
                FailureText.Text = "V URL chybí povinný parametr 'token'.";
                return;
            }

            BasicAuthService authService = new WebServices.BasicAuthService();
            BasicAuthService.AuthResult authResult = authService.ValidateToken(token);

            if (authResult == null)
            {
                FailureText.Text = "Neplatný uživatelský token.";
                return;

            }

            L_UserName.Text = authResult.UserName;
            L_Database.Text = authResult.DbName;

            if (authResult.Exception != null)
            {
                if (authResult.Exception is LoginException)
                {
                    FailureText.Text = KSProgram.Obecne.ReadGlobalResource.ZmenaTextu(authResult.Exception.Message);
                    return;
                }
                throw new Exception("Při autentizaci uživatele došlo k chybě", authResult.Exception);
            }

            Globals.Db = KSProgram.Data.DatabasePodleParam.Create(authResult.DbName, authResult.UserName, authResult.Password);

            try
            {
                KSProgram.SpravaUziv.InicializaceUzivatele(authResult.UserName);
            }
            catch (LoginException ex)
            {
                FailureText.Text = KSProgram.Obecne.ReadGlobalResource.ZmenaTextu(ex.Message);
                return;
            }

            if (Request.UrlReferrer != null)
                Session["LoginUrlReferrer"] = Request.UrlReferrer.ToString();
            System.Web.Security.FormsAuthentication.RedirectFromLoginPage(authResult.UserName, false);
        }
    }
}