﻿using System;
using System.Collections;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Linq;
using KSProgram.Data;

/// <summary>
/// Summary description for Opravneni
/// </summary>
[WebService(Namespace = "ksprtl/WebServices")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
// [System.Web.Script.Services.ScriptService]
public class Opravneni : System.Web.Services.WebService
{

    public Opravneni()
    {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    /// <summary>
    /// Vytvoří novou položku v číselníku oprávnění
    /// </summary>
    /// <param name="nazev">Název</param>
    /// <param name="popis">Popis</param>
    /// <param name="nadrizene_id">Nadřízené ID</param>
    /// <returns>ID nového záznamu nebo chybový stav</returns>
    [WebMethod(EnableSession = true)]
    public string Vytvorit(string nazev, string popis, int nadrizene_id)
    {
        if (string.IsNullOrEmpty(nazev))
            return "Název musí být zadán";
        else
            nazev = "'" + nazev + "'";

        if (string.IsNullOrEmpty(popis))
            popis = "null";
        else
            popis = "'" + popis + "'";

        int id = SqlPrikaz.ExecuteScalarInt32("select max(id_opr) from stromevpol_cis", 0);
        id++;

        string sqlInsert = "insert into stromevpol_cis (id_opr, polozka, popis, nadrizene_id)" +
            " values (" + id + "," + nazev + "," + popis + "," + nadrizene_id + ")";
        try
        {
            SqlPrikaz.ExecuteNonQuery(sqlInsert);
        }
        catch (Exception e)
        {
            return e.Message;
        }

        return id.ToString();
    }
    /// <summary>
    /// Aktualizuje položku v číselníku oprávnění
    /// </summary>
    /// <param name="id">ID aktualizovaného oprávnění</param>
    /// <param name="nazev">Nový název</param>
    /// <param name="popis">Nový popis</param>
    /// <param name="nadrazene_id">Nové nadřízené ID</param>
    /// <returns>OK nebo chybový stav</returns>
    [WebMethod(EnableSession = true)]
    public string Aktualizovat(int id, string nazev, string popis, string nadrazene_id)
    {
        if (string.IsNullOrEmpty(nazev))
            return "Název musí být zadán";
        else
            nazev = "'" + nazev + "'";

        if (string.IsNullOrEmpty(popis))
            popis = "null";
        else
            popis = "'" + popis + "'";

        string sqlUpdate = "update stromevpol_cis" +
            " set polozka = " + nazev + "," +
            " popis = " + popis + "," +
            " nadrizene_id = " + nadrazene_id +
            " where id_opr = " + id;
        try
        {
            SqlPrikaz.ExecuteNonQuery(sqlUpdate);
        }
        catch (Exception e)
        {
            return e.Message;
        }

        return "OK";
    }
    /// <summary>
    /// Přiřadí oprávnění vlastníkovi
    /// </summary>
    /// <param name="id_opravneni">ID oprávnění</param>
    /// <param name="typ_vazby">Typ vazby</param>
    /// <param name="id_vlastnika">ID vlastníka</param>
    /// <param name="platnost_od">Platnost vazby od</param>
    /// <param name="platnost_do">Platnost vazby do</param>
    /// <returns>OK nebo chybový stav</returns>
    [WebMethod(EnableSession = true)]
    public string Pridelit(int id_opravneni, string typ_vazby, string id_vlastnika, string platnost_od, string platnost_do)
    {
        //Kontrola vstupních hodnot
        //---------------------------
        int pocet = SqlPrikaz.ExecuteScalarInt32("select count(*) from stromevpol_cis where id_opr = " + id_opravneni, 0);
        if (pocet == 0) return "Oprávnění ID " + id_opravneni + " neexistuje.";

        DateTime datPom;
        if (DateTime.TryParse(platnost_od, out datPom))
            platnost_od = SqlPrikaz.DateTimeToString(datPom);
        else
            return "Neplatná hodnota parametru platnost_od: " + platnost_od;

        if (string.IsNullOrEmpty(platnost_do))
            platnost_do = "null";
        else
        {
            if (DateTime.TryParse(platnost_do, out datPom))
                platnost_do = SqlPrikaz.DateTimeToString(datPom);
            else
                return "Neplatná hodnota parametru platnost_do: " + platnost_do;
        }
        //---------------------------

        //Vytvoření vazby
        string sql = null;
        switch (typ_vazby)
        {
            case "Z":
                {
                    pocet = SqlPrikaz.ExecuteScalarInt32("select count(*) from pracovnici where os_cislo = " + id_vlastnika, 0);
                    if (pocet == 0) return "Neznámé osobní číslo: " + id_vlastnika;

                    //kontrola překrytí
                    //--------------------------
                    string sqlPrekryti = "select count(*) from stromevpol_osoby" +
                        " where id_opr = " + id_opravneni + " and os_cislo = " + id_vlastnika +
                        " and (plati_do is null or plati_do >= " + platnost_od + ")";
                    if (platnost_do != "null")
                        sqlPrekryti += " and plati_od <= " + platnost_do;
                    pocet = SqlPrikaz.ExecuteScalarInt32(sqlPrekryti, 0);
                    if (pocet > 0)
                        return "Os. číslo "+id_vlastnika+" již má v zadaném časovém intervalu"+
                            " přiděleno oprávnění ID "+id_opravneni;
                    //--------------------------

                    sql = "insert into stromevpol_osoby (id_opr, os_cislo, plati_od, plati_do)" +
                        " values (" + id_opravneni + "," + id_vlastnika + "," + platnost_od + "," + platnost_do + ")";
                    break;
                }
            case "SZ":
                {
                    pocet = SqlPrikaz.ExecuteScalarInt32("select count(*) from cis_skup_zam where id_skup_zam = " + id_vlastnika, 0);
                    if (pocet == 0) return "Neznámá skupina zaměstnanců ID: " + id_vlastnika;

                    //kontrola překrytí
                    //--------------------------
                    string sqlPrekryti = "select count(*) from stromevpol_skup_os" +
                        " where id_opr = " + id_opravneni + " and id_skup_zam = " + id_vlastnika +
                        " and (plati_do is null or plati_do >= " + platnost_od + ")";
                    if (platnost_do != "null")
                        sqlPrekryti += " and plati_od <= " + platnost_do;
                    pocet = SqlPrikaz.ExecuteScalarInt32(sqlPrekryti, 0);
                    if (pocet > 0)
                        return "Skupina zaměstnanců ID " + id_vlastnika + " již má v zadaném časovém intervalu" +
                            " přiděleno oprávnění ID " + id_opravneni;
                    //--------------------------

                    sql = "insert into stromevpol_skup_os (id_opr, id_skup_zam, plati_od, plati_do)" +
                        " values (" + id_opravneni + "," + id_vlastnika + "," + platnost_od + "," + platnost_do + ")";
                    break;
                }
            case "M":
                {
                    pocet = SqlPrikaz.ExecuteScalarInt32("select count(*) from prac_mist where id_pracmist = " + id_vlastnika, 0);
                    if (pocet == 0) return "Neznámé pracovncí místo ID: " + id_vlastnika;

                    //kontrola překrytí
                    //--------------------------
                    string sqlPrekryti = "select count(*) from stromevpol_pm" +
                        " where id_opr = " + id_opravneni + " and id_pracmist = " + id_vlastnika +
                        " and (plati_do is null or plati_do >= " + platnost_od + ")";
                    if (platnost_do != "null")
                        sqlPrekryti += " and plati_od <= " + platnost_do;
                    pocet = SqlPrikaz.ExecuteScalarInt32(sqlPrekryti, 0);
                    if (pocet > 0)
                        return "Pracovní místo ID " + id_vlastnika + " již má v zadaném časovém intervalu" +
                            " přiděleno oprávnění ID " + id_opravneni;
                    //--------------------------

                    sql = "insert into stromevpol_pm (id_opr, id_pracmist, plati_od, plati_do)" +
                        " values (" + id_opravneni + "," + id_vlastnika + "," + platnost_od + "," + platnost_do + ")";
                    break;
                }
            case "SM":
                {
                    pocet = SqlPrikaz.ExecuteScalarInt32("select count(*) from cis_skup_pm where id_skup_pm = " + id_vlastnika, 0);
                    if (pocet == 0) return "Neznámá skupina pracovncíh míst ID: " + id_vlastnika;

                    //kontrola překrytí
                    //--------------------------
                    string sqlPrekryti = "select count(*) from stromevpol_skup_pm" +
                        " where id_opr = " + id_opravneni + " and id_skup_pm = " + id_vlastnika +
                        " and (plati_do is null or plati_do >= " + platnost_od + ")";
                    if (platnost_do != "null")
                        sqlPrekryti += " and plati_od <= " + platnost_do;
                    pocet = SqlPrikaz.ExecuteScalarInt32(sqlPrekryti, 0);
                    if (pocet > 0)
                        return "Skupina pracovních míst ID " + id_vlastnika + " již má v zadaném časovém intervalu" +
                            " přiděleno oprávnění ID " + id_opravneni;
                    //--------------------------

                    sql = "insert into stromevpol_skup_pm (id_opr, id_skup_pm, plati_od, plati_do)" +
                        " values (" + id_opravneni + "," + id_vlastnika + "," + platnost_od + "," + platnost_do + ")";
                    break;
                }
            default:
                return "Neznámý typ vyzby: " + typ_vazby + ". Přípustné hodnoty jsou: Z - na zaměstnance, SZ - skupinu zaměstnanců, M - pracovní místo, SM - skupinu pracovních míst.";
        }

        if (sql != null)
        {
            try
            {
                SqlPrikaz.ExecuteNonQuery(sql);
            }
            catch (Exception e)
            {
                return e.Message;
            }
        }

        return "OK";
    }
    /// <summary>
    /// Odebere oprávnění
    /// </summary>
    /// <param name="id_opravneni">ID oprávnění</param>
    /// <param name="typ_vazby">Typ vazby</param>
    /// <param name="id_vlastnika">ID vlastníka</param>
    /// <param name="platnost_do">Platnost vazby do</param>
    /// <returns>OK nebo chybový stav</returns>
    [WebMethod(EnableSession = true)]
    public string Odebrat(int id_opravneni, string typ_vazby, string id_vlastnika, string platnost_do)
    {
        //Kontrola vstupních hodnot
        //---------------------------
        int pocet = SqlPrikaz.ExecuteScalarInt32("select count(*) from stromevpol_cis where id_opr = " + id_opravneni, 0);
        if (pocet == 0) return "Oprávnění ID " + id_opravneni + " neexistuje.";

        DateTime datPom;
        if (DateTime.TryParse(platnost_do, out datPom))
            platnost_do = SqlPrikaz.DateTimeToString(datPom);
        else
            return "Neplatná hodnota parametru platnost_do: " + platnost_do;
        //---------------------------

        //Zrušení vazby
        string sql = null;
        switch (typ_vazby)
        {
            case "Z":
                {
                    sql = "update stromevpol_osoby set plati_do = " + platnost_do +
                            " where id_opr = " + id_opravneni + " and os_cislo = " + id_vlastnika +
                            " and (plati_do is null or plati_do > " + platnost_do + ")";
                    break;
                }
            case "SZ":
                {
                    sql = "update stromevpol_skup_os set plati_do = " + platnost_do +
                            " where id_opr = " + id_opravneni + " and id_skup_zam = " + id_vlastnika +
                            " and (plati_do is null or plati_do > " + platnost_do + ")";
                    break;
                }
            case "M":
                {
                    sql = "update stromevpol_pm set plati_do = " + platnost_do +
                            " where id_opr = " + id_opravneni + " and id_pracmist = " + id_vlastnika +
                            " and (plati_do is null or plati_do > " + platnost_do + ")";
                    break;
                }
            case "SM":
                {
                    sql = "update stromevpol_skup_pm set plati_do = " + platnost_do +
                            " where id_opr = " + id_opravneni + " and id_skup_pm = " + id_vlastnika +
                            " and (plati_do is null or plati_do > " + platnost_do + ")";
                    break;
                }
            default:
                return "Neznámý typ vyzby: " + typ_vazby + ". Přípustné hodnoty jsou: Z - na zaměstnance, SZ - skupinu zaměstnanců, M - pracovní místo, SM - skupinu pracovních míst.";
        }

        if (sql != null)
        {
            try
            {
                SqlPrikaz.ExecuteNonQuery(sql);
            }
            catch (Exception e)
            {
                return e.Message;
            }
        }

        return "OK";
    }
}

