﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Xml.Serialization;
using KSProgram;
using KSProgram.Sluzebky;
using KSProgram.Pozadavky;
using KSProgram.Funkce;
using KSProgram.Data;
using System.Data.Common;
using KSProgram.Security;
using System.IO;
using KSProgram.Diagnostics;
using KSProgram.Obecne;
using System.Web.Security;
using System.Web.Services.Protocols;
using System.Net.Mail;

namespace KSPortal.WebServices
{
    /// <summary>
    /// Summary description for SCesty
    /// </summary>
    [WebService(Namespace = "ksprtl/WebServices")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class SCesty : System.Web.Services.WebService
    {
        public AuthKeyHeader AuthHeader;

        /// <summary>
        /// Proceed authentication
        /// </summary>
        /// <param name="auth">Authentication object</param>
        /// <returns>Error message</returns>
        private string Authentication(AuthKeyHeader auth)
        {
            string configKey = KSProgram.Funkce.AppConfiguration.GetAppSetting<string>("WsAuthKey", null);

            if (Zakaznik.IsCurrent("VARROC") && auth == null && configKey == null) return null;

            if (auth == null) return "Autentizace: metoda musí být volána s autentizační SOAP hlavičkou AuthKeyHeader.";
            if (configKey == null) return "Autentizace: není nastaven autentizační klíč AppSettings/WsAuthKey v konfiguračním souboru KS portálu.";
            if (auth.AuthKey != configKey)
                return "Autentizace: nesprávný klíč.";
            return null;
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="cesta">Třída s daty služební cesty včetně podčástí</param>
        /// <returns></returns>
        [WebMethod(EnableSession = true, Description = "Import služební cesty.")]
        [SoapHeader("AuthHeader")]
        public SCResult CestaVloz(SCCesta c)
        {
            /*this.Context.Request.InputStream.Position = 0;
            StreamReader stream = new StreamReader(this.Context.Request.InputStream);
            string x = stream.ReadToEnd();
            string xml = HttpUtility.UrlDecode(x);
            xml += "";*/

            SCResult r = new SCResult();
            r.ErrorCode = -99;
            r.ErrorString = "Chyba";

            try
            {
                //r.ErrorString = r.ErrorString.Substring(255); //test chyby

                //PRIPOJENI K DB
                if (null == Globals.Db)
                {
                    Globals.Db = DatabasePodleParam.Create(null);
                    KSTrace.WriteLine("Aktivní DB", Globals.Db.ConnectionStringWithoutCredentials);
                }

                //AUTENTIZACE
                string authMessage = Authentication(AuthHeader);
                if (authMessage != null)
                {
                    r.ErrorCode = -401;
                    r.ErrorString = authMessage;
                    return r;
                }

                //VALIDACE HODNOT
                //PR 11.2.2015 uvolněno pro VUANCH
                if (!(Zakaznik.IsCurrent("VARROC") || Zakaznik.IsCurrent("VUANCH")))
                {
                    r.ErrorCode = -3;
                    //PR 28.7.2016 r.ErrorString = "Metoda není uvolněna pro vaší organizaci.";
                    r.ErrorString = "Používáte starou verzi webové služby, použijte verzi 2.";
                    return r;
                }

                bool pouzitWorkFlow = (c.WorkFlow != null && c.WorkFlow.Count > 0);
                workfl_stavy lo_validniStavy = null;

                c.Cislo = Trunc(c.Cislo, 25);

                Decimal os_cislo = OsobaNaOsCislo(c.Zamestnanec);
                String zdroj = "E";
                if (String.IsNullOrEmpty(c.PocMisto)) c.PocMisto = SlcHlavicka.VychoziMistoOdjezdu(os_cislo);
                if (String.IsNullOrEmpty(c.KonMisto)) c.KonMisto = c.PocMisto;

                String spolucestujici = "";
                decimal?[] spolu = new decimal?[10];
                if (c.Spolucestujici != null && c.Spolucestujici.Count > 0)
                {
                    for (int i = 0; i < Math.Min(10, c.Spolucestujici.Count); i++)
                    {
                        spolu[i] = OsobaNaOsCislo(c.Spolucestujici[i]);
                        if (spolucestujici.Length > 0) spolucestujici += ", ";
                        if (spolu[i] != null)
                            spolucestujici += SlcHlavicka.ZkraceneJmeno(spolu[i].Value);
                        else
                            spolucestujici += c.Spolucestujici[i].Login;
                    }
                    spolucestujici = Trunc(spolucestujici, 120);
                }

                if (null == c.Zamestnanec)
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Není zadán zaměstnanec (SCCesta.Zamestnanec).";
                    return r;
                }
                if (!Pracovnici.ExistOsCislo(os_cislo))
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Zaměstnanec s os. číslem " + os_cislo + " neexistuje.";
                    return r;
                }
                Globals.CreateRadPrava(os_cislo/*, DateTime.Now*/); //PR 7.5.2015 aby se později nevytvářely pro os_cislo=0, což by hodilo vyjímku
                if (c.DatOd.Year < 2000 || c.DatOd.Year > 2100)
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Není zadán datum počátku SC (SCCesta.DatOd).";
                    return r;
                }
                if (c.DatDo.Year < 2000 || c.DatDo.Year > 2100)
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Není zadán datum konce SC (SCCesta.DatDo).";
                    return r;
                }
                if (String.IsNullOrEmpty(c.Cil))
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Není zadán cíl (SCCesta.Cil).";
                    return r;
                }
                c.Cil = Trunc(c.Cil, 80);
                if (String.IsNullOrEmpty(c.Ucel))
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Není zadán účel (SCCesta.Ucel).";
                    return r;
                }
                c.Ucel = Trunc(c.Ucel, 80);
                c.PocMisto = Trunc(c.PocMisto, 40);
                c.KonMisto = Trunc(c.KonMisto, 40);
                if (String.IsNullOrEmpty(c.Typ))
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Není zadán typ SC (SCCesta.Typ).";
                    return r;
                }
                c.Typ = c.Typ.ToLower();
                if (c.Typ != "t" && c.Typ != "z")
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Neplatný typ SC (SCCesta.Typ), validní hodnoty jsou t a z.";
                    return r;
                }
                if (String.IsNullOrEmpty(c.DruhDopr))
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Není zadán Druh dopr. prostředku (SCCesta.DruhDopr).";
                    return r;
                }
                else //PR 19.1.2015 opravena kontrola, aby počítala s více druhy oddělenými čárkou
                {
                    string[] ls_d = c.DruhDopr.Split(',');
                    foreach (string d in ls_d)
                    {
                        if (String.IsNullOrEmpty(SlcCisDruhDopr.Nazev(d)))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Neplatný druh dopr. prostředku " + d + " (SCCesta.DruhDopr).";
                            return r;
                        }
                    }
                }

                //validace zaloh
                if (c.Zalohy != null) //PR 12.2.2018 
                    foreach (SCZaloha z in c.Zalohy)
                    {
                        if (!String.IsNullOrEmpty(z.Druh))
                        {
                            if (String.IsNullOrEmpty(SlcCisCn.Nazev(z.Druh)))
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Neplatný druh zálohy " + z.Druh + " (SCZaloha.Druh).";
                                return r;
                            }
                        }
                        if (!CisKodMeny.Existuje(z.KodMeny))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Neplatný kód měny zálohy " + z.KodMeny + " (SCZaloha.KodMeny).";
                            return r;
                        }
                        if (!String.IsNullOrEmpty(z.VyplJak))//PR 31.7.2014 
                        {
                            if (String.IsNullOrEmpty(SlcCisZpusPlat.Nazev(z.VyplJak)))
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Neplatný způsob vyplacení zálohy " + z.VyplJak + " (SCZaloha.VyplJak).";
                                return r;
                            }
                        }
                        z.VyplDoklad = Trunc(z.VyplDoklad, 15);
                    }

                //validace prubehu
                if (c.Prubeh != null) //PR 12.2.2018 
                    foreach (SCPrubeh p in c.Prubeh)
                    {
                        if (p.DatOd < c.DatOd || p.DatOd > c.DatDo || p.DatDo < c.DatOd || p.DatDo > c.DatDo)
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Odjezd nebo příjezd v části průběh není v intervalu konání SC (SCPrubeh.DatOd/DatDo).";
                            return r;
                        }
                        if (String.IsNullOrEmpty(p.MistoOd))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Není zadáno místo odjezdu (SCPrubeh.MistoOd).";
                            return r;
                        }
                        p.MistoOd = Trunc(p.MistoOd, 40);
                        if (String.IsNullOrEmpty(p.MistoDo))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Není zadáno místo příjezdu (SCPrubeh.MistoDo).";
                            return r;
                        }
                        p.MistoDo = Trunc(p.MistoDo, 40);
                        if (!String.IsNullOrEmpty(p.ZemeOd))
                        {
                            if (!SlcCisZemi.Existuje(p.ZemeOd))
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Neplatný název země " + p.ZemeOd + " (SCPrubeh.ZemeOd).";
                                return r;
                            }
                            p.ZemeOd = Trunc(p.ZemeOd, 45);
                        }
                        if (!String.IsNullOrEmpty(p.ZemeDo))
                        {
                            if (!SlcCisZemi.Existuje(p.ZemeDo))
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Neplatný název země " + p.ZemeDo + " (SCPrubeh.ZemeDo).";
                                return r;
                            }
                            p.ZemeDo = Trunc(p.ZemeDo, 45);
                        }
                        if (String.IsNullOrEmpty(p.DruhDopr)) //PR 21.7.2016 c.DruhDopr -> p.DruhDopr
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Není zadán Druh dopr. prostředku (SCPrubeh.DruhDopr).";
                            return r;
                        }
                        else if (String.IsNullOrEmpty(SlcCisDruhDopr.Nazev(p.DruhDopr)))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Neplatný druh dopr. prostředku " + p.DruhDopr + " (SCPrubeh.DruhDopr).";
                            return r;
                        }
                    }
                //validace workflow //PR 7.8.2014 
                if (pouzitWorkFlow)
                {
                    lo_validniStavy = new workfl_stavy(5);
                    foreach (SCWorkflowProc p in c.WorkFlow)
                    {
                        if (String.IsNullOrEmpty(p.Stav))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Není zadán Stav (SCWorkflowProc.Stav).";
                            return r;
                        }
                        //PR 11.2.2015 zrušena dočasná vyjímka
                        //else if (p.Stav.Equals("povoleno"))
                        //    p.Stav = "povolena";
                        bool lb_valid = false;
                        foreach (Str_workfl_stavy s in lo_validniStavy.Stavy)
                        {
                            if (s.stav.Equals(p.Stav))
                            {
                                lb_valid = true;
                                break;
                            }
                        }
                        if (!lb_valid)
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Neplatný druh stavu " + p.Stav + " (SCWorkflowProc.Stav).";
                            return r;
                        }

                        /*if (!(p.Stav.Equals("novy") || p.Stav.Equals("povolena")))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Neplatný druh stavu " + p.Stav + ", podporované jsou pouze novy a povolena (SCWorkflowProc.Stav).";
                            return r;
                        }*/
                        if (p.Resitel == null)
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Není zadán Resitel (SCWorkflowProc.Resitel), pro ne-uživatele portálu nastav Resitel.OsCislo=0.";
                            return r;
                        }
                        else if (p.Resitel.OsCislo == null && String.IsNullOrEmpty(p.Resitel.Login))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Není zadán Resitel.OsCislo ani Resitel.Login (SCWorkflowProc.Resitel), pro ne-uživatele portálu nastav Resitel.OsCislo=0.";
                            return r;
                        }
                    }
                }

                r.ErrorCode = -4;

                bool lb_vlastni, lb_sluzebni;
                string ls_spz, ls_phm = "", ls_pov_ruc = "", ls_druh = "";
                decimal ldc_vlastnik = 0, ldc_prum_spotreba = 0, ldc_spotreba_mesto = 0, ldc_spotreba_mimo = 0;
                //PR 10.11.2015 //decimal ldc_zakl_nahr = 0, ldc_cena_phm = 0;
                ls_spz = c.Spz;
                lb_vlastni = c.DruhDopr.Contains("AUV") || c.DruhDopr.Contains("NAV") || c.DruhDopr.Contains("MOV");
                lb_sluzebni = c.DruhDopr.Contains("AUS") || c.DruhDopr.Contains("NAS") || c.DruhDopr.Contains("MOS");
                //if (lb_vlastni) 
                //PR 9.11.2017 
                if (lb_vlastni || (lb_sluzebni && Parametr_cti.Get("VuzSluzSoukPHM", DateTime.Today, ViceIco.GetIdPodnikuZamestnance(os_cislo)) == "a"))
                {                    
                    SlcVozidla.NactiInfoOvozidle(ls_spz, out ldc_vlastnik, out ls_druh, out ls_phm, out ldc_spotreba_mesto, out ldc_spotreba_mimo, out ldc_prum_spotreba, out ls_pov_ruc);
                    //ldc_zakl_nahr = KSProgram.Sluzebky.SlcSazby.of_zjistiSazbuZaklNahr(c.DatOd, ls_druh);
                    //ldc_cena_phm = KSProgram.Sluzebky.SlcCisPhm.Cena(ls_phm, c.DatDo);
                }

                //if (ldc_vlastnik > 0 && ldc_vlastnik != os_cislo)
                //PR 9.11.2017 
                if (ldc_vlastnik != os_cislo)
                {//PR 13.12.2012 pokud neni vlastnikem, tak to vse nastavim na null
                    ldc_spotreba_mesto = 0;
                    ldc_spotreba_mimo = 0;
                    ldc_prum_spotreba = 0;
                    ls_phm = "";
                    //ldc_cena_phm = 0;
                    //ldc_zakl_nahr = 0;
                }

                decimal ldc_povolil = OsobaNaOsCislo(c.Povolil);

                r.ErrorCode = -5;

                using (DbConnection connection = SqlPrikaz.CreateConnectionWithTrans())
                {
                    try
                    {
                        String tabulka = pouzitWorkFlow ? "workfl_sluz_cesty" : "sluz_cesty";
                        Int32 ev_cislo = 0;
                        bool lb_update = false;
                        if (!String.IsNullOrEmpty(c.Cislo))
                        {
                            ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from sluz_cesty where cislo='" + c.Cislo + "' and os_cislo=" + os_cislo, 0);
                            if (ev_cislo > 0)
                                tabulka = "sluz_cesty";
                            else
                            {
                                ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from workfl_sluz_cesty where cislo='" + c.Cislo + "' and os_cislo=" + os_cislo, 0);
                                if (ev_cislo > 0)
                                    tabulka = "workfl_sluz_cesty";
                            }

                            if (ev_cislo > 0)
                            {
                                lb_update = true;
                                //r.ErrorCode = -2;
                                //r.ErrorString = "Záznam pro zaměstnance s Číslem služební cesty " + c.Cislo + " již existuje";
                                //return r;
                            }
                        }
                        r.ErrorCode = -6;
                        if (lb_update)
                        {
                            KSProgram.Sluzebky.SlcHlavicka.Update(tabulka, ev_cislo, ev_cislo, null, c.Cislo, os_cislo, c.Typ,
                                c.PocMisto, c.DatOd.Date, c.DatOd.ToString("HH:mm"), c.KonMisto, c.DatDo.Date, c.DatDo.ToString("HH:mm"),
                                c.Cil, String.Empty, c.Ucel,
                                spolucestujici, spolu[0], spolu[1], spolu[2], spolu[3], spolu[4], spolu[5], spolu[6], spolu[7], spolu[8], spolu[9],
                                c.DruhDopr, c.Spz, /*"", "", PR 21.05.2019 zruseny arg spz_sluz,spz_vlas*/ /*ldc_zakl_nahr,*/
                                ls_phm, /*ldc_cena_phm,*/ ldc_prum_spotreba, ldc_spotreba_mesto, ldc_spotreba_mimo, ls_pov_ruc, null, null, null, null, null, null, null);
                            
                            if (c.Zalohy!=null) KSProgram.Sluzebky.SlcZalohy.DeleteAll(tabulka, ev_cislo, /*i_vyplacene*/ false);
                            if (c.Prubeh != null) KSProgram.Sluzebky.SlcPrubeh.DeleteAll(ev_cislo);

                        }
                        else
                        {
                            KSProgram.Sluzebky.SlcHlavicka.Insert(tabulka, out ev_cislo, null, os_cislo, zdroj, c.Typ,
                                c.PocMisto, c.DatOd.Date, c.DatOd.ToString("HH:mm"), c.KonMisto, c.DatDo.Date, c.DatDo.ToString("HH:mm"),
                                c.Cil, String.Empty, c.Ucel,
                                spolucestujici, spolu[0], spolu[1], spolu[2], spolu[3], spolu[4], spolu[5], spolu[6], spolu[7], spolu[8], spolu[9],
                                c.DruhDopr, c.Spz, /*"", "", PR 21.05.2019 zruseny arg spz_sluz,spz_vlas*/ /*ldc_zakl_nahr,*/
                                ls_phm, /*ldc_cena_phm,*/ ldc_prum_spotreba, ldc_spotreba_mesto, ldc_spotreba_mimo, ls_pov_ruc,
                                c.DatumSouhlasu, c.PovolilDne, ldc_povolil, null, null, null, null, null, null, null);

                            if (!String.IsNullOrEmpty(c.Cislo))
                            {   //zmena cisla SC
                                SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set cislo='" + c.Cislo + "' where ev_cislo=" + ev_cislo);
                                //PR 10.7.2015 zapis cisla do dbarchu, aby se u smazane SC dalo dohledat ev_cislo dle externiho cisla
                                /*KM 7.10.2015 string podm = "ev_cislo=" + ev_cislo + " and os_cislo=" + os_cislo;
                                DBArch.Insert(tabulka, "cislo", ev_cislo.ToString(), c.Cislo, podm, "SCesty.CestaVloz");*/
                            }
                            else
                                c.Cislo = ev_cislo.ToString();
                        }
                        r.ErrorCode = -7;
                        if (c.Vytvoril != null && (!String.IsNullOrEmpty(c.Vytvoril.Login) || c.Vytvoril.OsCislo != null))
                        {
                            string ls_vytvoril;
                            if (!String.IsNullOrEmpty(c.Vytvoril.Login))
                                ls_vytvoril = c.Vytvoril.Login;
                            else
                            {
                                if (c.Vytvoril.OsCislo == null || c.Vytvoril.OsCislo == 0)
                                    ls_vytvoril = "system";
                                else
                                    ls_vytvoril = SpravaUziv.UzivatelPodleOsobnihoCisla(c.Vytvoril.OsCislo.Value);
                            }
                            SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set vytvoril=" + SqlPrikaz.ValueToString(ls_vytvoril)
                                    + ", vytvor_dne=" + SqlPrikaz.ValueToString(c.VytvorDne)
                                    + " where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));
                        }
                        r.ErrorCode = -8;
                        if (c.Zalohy != null && c.Zalohy.Count > 0)
                        {
                            //PR 9.3.2017 
                            if (lb_update) SlcZalohy.DeleteAll(ev_cislo, /*i_vyplacene*/ false); //PR 9.3.2017 mažu jen nevyplacené zálohy
                            string tab_zal;
                            if (tabulka == "workfl_sluz_cesty")
                                tab_zal = "workfl_slc_zalohy";
                            else
                                tab_zal = "slc_zalohy";
                            foreach (SCZaloha z in c.Zalohy)
                            {
                                int zalId = 0;
                                DateTime? vypl_dne = null;
                                string vypl_jak = null;
                                zalId = SlcZalohy.Find(tab_zal, ev_cislo, z.Castka, z.KodMeny, z.Druh, out vypl_dne, out vypl_jak);//PR 28.3.2017 ev_cislo
                                if (z.VyplJak == "") z.VyplJak = null;
                                if (vypl_jak == "") vypl_jak = null;
                                if (zalId == 0)
                                    SlcZalohy.Insert(tab_zal, ev_cislo, out zalId, z.Druh, z.Castka, z.KodMeny,
                                        z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), 
                                        z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad, null);
                                else if (zalId > 0 && (vypl_dne == null || (z.VyplatilDne != null && vypl_dne == z.VyplatilDne)) && (vypl_jak == z.VyplJak || (vypl_jak == null && z.VyplJak != null)))
                                    //PR 9.3.2017 jen pokud není vyplacena nebo se nemění datum vyplacení,
                                    //a zároveň se nemění způsob vyplacení z jednoho typu na jiný, tak ji updatnu (jinak ji ponechám bezezměny)
                                    SlcZalohy.Update(tab_zal, zalId, zalId, ev_cislo, z.Druh, z.Castka, z.KodMeny,
                                        z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), 
                                        z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad, null);
                            }
                        }
                        r.ErrorCode = -9;
                        if (pouzitWorkFlow)
                        {
                            int id_poz = 0;
                            int? id_poztmp = Pozadaveksluzc.EvCisloToIdPozadavku(ev_cislo);
                            if (id_poztmp!=null && id_poztmp > 0) id_poz = id_poztmp.Value;
                            if (id_poz == 0)
                                PozadavekInsert(ref id_poz, os_cislo, c.DatOd, c.DatDo, c.Cil, c.Ucel, c.DruhDopr, tabulka, ev_cislo, null);
                            
                            //Pozadaveksluzc p = new Pozadaveksluzc();
                            //p.Retrieve(idPoz);
                            int posledni = c.WorkFlow.Count - 1;
                            for (int poradi = 0; poradi < c.WorkFlow.Count; poradi++)
                            {
                                
                                SCWorkflowProc p = c.WorkFlow[poradi];
                                decimal ldc_resitel = OsobaNaOsCislo(p.Resitel);
                                                                
                                Str_workfl_proc str = new Str_workfl_proc();
                                str.id_poz = id_poz;
                                str.poradi = poradi + 1;
                                
                                foreach (Str_workfl_stavy st in lo_validniStavy.Stavy)
                                {
                                    if (st.stav.Equals(p.Stav))
                                    {
                                        str.faze = st.faze;
                                        str.stav = st.stav;
                                        str.substav = "";
                                        str.sys_stav = st.sys_stav;
                                        if (poradi == posledni) //PR 7.5.2015 
                                            //str.adresat = WorkflAdresat.UpravAdresataProWorkflProc(id_poz, st.adresat);//PR 11.2.2015 
                                            str.adresat = ""; //PR 3.11.2017 Adresát se vezme dle nastavení
                                        else//PR 7.5.2015 
                                            str.adresat = "[oc=" + OsobaNaOsCislo(c.WorkFlow[poradi + 1].Resitel) + "]";
                                        break;
                                    }
                                }
                                
                                str.resitel = ldc_resitel;
                                str.datum = p.Datum;                                
                                str.duvod = p.Duvod;

                                if (WfProcInsert(str, /*sendmail*/poradi == posledni && str.datum == DateTime.Today))
                                {

                                    //if (p.Stav == "novy" && os_cislo == ldc_resitel)
                                    //{
                                    //    SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set souhl_dne=" + SqlPrikaz.ValueToString(p.Datum)
                                    //    + " where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));
                                    //}
                                    //if (p.Stav == "povolena")
                                    //{
                                    //    SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set povolil_osc=" + SqlPrikaz.ValueToString(ldc_resitel)
                                    //    + ", povolil_dne=" + SqlPrikaz.ValueToString(p.Datum)
                                    //    + " where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));

                                    //    if (tabulka == "workfl_sluz_cesty")
                                    //    {
                                    //        tabulka = "sluz_cesty";
                                    //        SqlPrikaz.ExecuteNonQuery("insert into sluz_cesty select * from workfl_sluz_cesty where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));
                                    //        SqlPrikaz.ExecuteNonQuery("update workfl_body set tabulka='sluz_cesty' where archiv is null and id_poz=" + SqlPrikaz.ValueToString(id_poz));
                                    //    }
                                    //}
                                    ////PR 11.2.2015 odeslání mailu pro poslední krok
                                    //if (poradi == posledni && lb_update == false && !Zakaznik.IsCurrent("VARROC"))
                                    //{
                                    //    WorkflowProces.CreateAndSendMail(id_poz);
                                    //}
                                }
                            }
                        }//WF

                        //if (c.Zalohy != null && c.Zalohy.Count > 0)
                        //{
                        //    r.ErrorCode = -9;
                        //    //if (lb_update) SlcZalohy.DeleteAll(ev_cislo);
                        //    //foreach (SCZaloha z in c.Zalohy)
                        //    //{
                        //    //    int zalId;
                        //    //    SlcZalohy.Insert(tabulka, ev_cislo, out zalId, z.Druh, z.Castka, z.KodMeny,
                        //    //        z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad);
                        //    //}
                        //    //PR 9.3.2017 
                        //    if (lb_update) SlcZalohy.DeleteAll(ev_cislo, /*i_vyplacene*/ false); //PR 9.3.2017 mažu jen nevyplacené zálohy
                        //    string tab_zal;
                        //    if (tabulka == "workfl_sluz_cesty")
                        //        tab_zal = "workfl_slc_zalohy";
                        //    else
                        //        tab_zal = "slc_zalohy";
                        //    foreach (SCZaloha z in c.Zalohy)
                        //    {
                        //        int zalId = 0;
                        //        DateTime? vypl_dne = null;
                        //        string vypl_jak = null;
                        //        zalId = SlcZalohy.Find(tab_zal, ev_cislo, z.Castka, z.KodMeny, z.Druh, out vypl_dne, out vypl_jak);//PR 28.3.2017 ev_cislo
                        //        if (z.VyplJak == "") z.VyplJak = null;
                        //        if (vypl_jak == "") vypl_jak = null;
                        //        if (zalId == 0)
                        //            SlcZalohy.Insert(tab_zal, ev_cislo, out zalId, z.Druh, z.Castka, z.KodMeny,
                        //                z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad);
                        //        else if (zalId > 0 && (vypl_dne == null || (z.VyplatilDne != null && vypl_dne == z.VyplatilDne)) && (vypl_jak == z.VyplJak || (vypl_jak == null && z.VyplJak != null)))
                        //            //PR 9.3.2017 jen pokud není vyplacena nebo se nemění datum vyplacení,
                        //            //a zároveň se nemění způsob vyplacení z jednoho typu na jiný, tak ji updatnu (jinak ji ponechám bezezměny)
                        //            SlcZalohy.Update(tab_zal, zalId, zalId, ev_cislo, z.Druh, z.Castka, z.KodMeny,
                        //                z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad);
                        //    }
                        //}

                        if (c.Prubeh != null && c.Prubeh.Count>0)
                        {
                            r.ErrorCode = -10;
                            if (lb_update) SlcPrubeh.DeleteAll(ev_cislo);
                            int ll_r=0;
                            foreach (SCPrubeh p in c.Prubeh)
                            {
                                //SlcPrubeh.Insert(ev_cislo, p.DatOd, p.DatDo, p.MistoOd, p.MistoDo, p.ZemeOd, p.ZemeDo, p.DatOd.ToString("HH:mm"),
                                //    p.DatDo.ToString("HH:mm"), p.DruhDopr, p.KmMesto, p.KmMimo, p.KmKomb, p.KmCelkem,
                                //    p.PracVykonOd.HasValue ? p.PracVykonOd.Value.ToString("HH:mm") : "",
                                //    p.PracVykonDo.HasValue ? p.PracVykonDo.Value.ToString("HH:mm") : "",
                                //    p.HraniceDat, p.HraniceDat.HasValue ? p.HraniceDat.Value.ToString("HH:mm") : "", p.HraniceMisto,
                                //    p.Preruseni.HasValue ? p.Preruseni.Value : false);
                                //PR 5.2.2016 slc_prubeh -> slc_prub
                                if (p.Preruseni.HasValue && p.Preruseni.Value == true)
                                {
                                    ll_r++;
                                    SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, "R", p.DatOd, p.DatOd.ToString("HH:mm"), p.MistoOd, p.ZemeOd, p.ZemeOd, "", 0, 0, 0, 0);
                                    ll_r++;
                                    SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, "U", p.DatDo, p.DatDo.ToString("HH:mm"), p.MistoDo, p.ZemeDo, p.ZemeDo, "", 0, 0, 0, 0);
                                }
                                else
                                {
                                    ll_r++;
                                    SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, "O", p.DatOd, p.DatOd.ToString("HH:mm"), p.MistoOd, p.ZemeOd, p.ZemeOd, p.DruhDopr, 0, 0, 0, 0);
                                    if (p.HraniceDat.HasValue)
                                    {
                                        ll_r++;
                                        SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, "H", p.HraniceDat, p.HraniceDat.Value.ToString("HH:mm"), p.HraniceMisto, p.ZemeOd, p.ZemeDo, p.DruhDopr, 0, 0, 0, 0);
                                    }
                                    ll_r++;
                                    SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, "P", p.DatDo, p.DatDo.ToString("HH:mm"), p.MistoDo, p.ZemeDo, p.ZemeDo, p.DruhDopr, p.KmMesto, p.KmMimo, p.KmKomb, p.KmCelkem);

                                    if (p.PracVykonOd.HasValue && p.PracVykonDo.HasValue)
                                    {
                                        ll_r++;//PR 17.01.2022 novy sloupec poradi
                                        SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, "V", p.PracVykonOd, p.PracVykonOd.Value.ToString("HH:mm"), "", "", "", "", 0, 0, 0, 0);
                                        ll_r++;
                                        SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, "K", p.PracVykonDo, p.PracVykonDo.Value.ToString("HH:mm"), "", "", "", "", 0, 0, 0, 0);
                                    }
                                }
                            }
                        }

                        if (c.Vydaje != null && c.Vydaje.Count > 0)
                        {
                            r.ErrorCode = -11;
                            if (lb_update) SlcNahrady.DeleteAll(ev_cislo);
                            foreach (SCVydaj v in c.Vydaje)
                            {
                                int vydId;
                                SlcNahrady.Insert(out vydId, ev_cislo, v.Datum, v.Druh, v.ZpusPlat, null, v.Castka, v.KodMeny, null);
                            }
                        }

                        SqlPrikaz.Commit();
                    }
                    catch (Exception ex)
                    {
                        SqlPrikaz.Rollback();
                        //r.ErrorCode = -1;
                        r.ErrorString = ex.Message;
                        if (null != ex.InnerException) r.ErrorString += ";\r\nInnerException=" + ex.InnerException.Message;
                        if (null != ex.Source) r.ErrorString += ";\r\nSource=" + ex.Source;
                        if (null != ex.StackTrace) r.ErrorString += ";\r\nStackTrace=" + ex.StackTrace;
                        return r;
                    }    
                }
            }
            catch (Exception ex)
            {
                //r.ErrorCode = -1;
                r.ErrorString = ex.Message;
                if (null != ex.InnerException) r.ErrorString += ";\r\nInnerException=" + ex.InnerException.Message;
                if (null != ex.Source) r.ErrorString += ";\r\nSource=" + ex.Source;
                if (null != ex.StackTrace) r.ErrorString += ";\r\nStackTrace=" + ex.StackTrace;
                return r;
            }
            r.ErrorCode = 0;
            r.ErrorString = "OK";
            if (!Zakaznik.IsCurrent("VARROC")) r.ErrorString += " (Cislo="+c.Cislo+")";
            return r;
        }

        /// <summary>
        /// Smazání služební cesty.
        /// </summary>
        /// <param name="cislo"></param>
        /// <returns></returns>
        [WebMethod(EnableSession = true, Description = "Smazání služební cesty.")]
        [SoapHeader("AuthHeader")]
        public SCResult CestaSmaz(string Cislo, decimal OsCislo)
        {
            SCResult r = new SCResult();
            r.ErrorCode = -99;
            r.ErrorString = "Chyba";

            try
            {
                //PRIPOJENI K DB
                if (null == Globals.Db)
                {
                    Globals.Db = DatabasePodleParam.Create(null);
                    KSTrace.WriteLine("Aktivní DB", Globals.Db.ConnectionStringWithoutCredentials);
                }

                //AUTENTIZACE
                //if (!this.User.Identity.IsAuthenticated)
                //string at = this.User.Identity.AuthenticationType == "" ? "anonymous" : this.User.Identity.AuthenticationType;
                string authMessage = Authentication(AuthHeader);
                if (authMessage != null)
                {
                    r.ErrorCode = -401;
                    r.ErrorString = authMessage;
                    return r;
                }

                //VALIDACE HODNOT
                //PR 11.2.2015 uvolněno pro VUANCH
                if (!(Zakaznik.IsCurrent("VARROC") || Zakaznik.IsCurrent("BLANKYT") || Zakaznik.IsCurrent("VUANCH")))
                {
                    r.ErrorCode = -3;
                    r.ErrorString = "Metoda není uvolněna pro vaší organizaci.";
                    return r;
                }
                                
                if (String.IsNullOrEmpty(Cislo))
                {
                    r.ErrorCode = -11;
                    r.ErrorString = "Není zadáno číslo SC.";
                    return r;
                }

                if (OsCislo <= 0)
                {
                    r.ErrorCode = -12;
                    r.ErrorString = "Není zadáno osobní číslo.";
                    return r;
                }

                int ev_cislo = 0;
                ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from sluz_cesty where cislo='" + Cislo + "' and os_cislo=" + OsCislo, 0);
                if (ev_cislo == 0)
                    ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from workfl_sluz_cesty where cislo='" + Cislo + "' and os_cislo=" + OsCislo, 0);
                    

                if (ev_cislo == 0)
                {
                    r.ErrorCode = -13;
                    r.ErrorString = "SC nenalezena (cislo='" + Cislo + "' and os_cislo=" + OsCislo + ").";
                    return r;
                }

                if (SlcFunkce.JeVyuctovana(ev_cislo)) //PR 16.10.2014 
                {
                    r.ErrorCode = -14;
                    r.ErrorString = "Nelze smazat: cestovní příkaz již byl vyúčtován!";
                    return r;
                }
                else
                {
                    int? id_poz = Pozadaveksluzc.EvCisloToIdPozadavku(ev_cislo);
                    if (id_poz.HasValue && id_poz.Value > 0)
                    {
                        Pozadaveksluzc poz = new Pozadaveksluzc();
                        poz.DeleteBase(id_poz.Value);                        
                    }
                    else
                        SlcFunkce.DeleteAllData(ev_cislo);
                }
                
            }
            catch (Exception ex)
            {
                //r.ErrorCode = -1;
                r.ErrorString = ex.Message;
                if (null != ex.InnerException) r.ErrorString += ";\r\nInnerException=" + ex.InnerException.Message;
                if (null != ex.Source) r.ErrorString += ";\r\nSource=" + ex.Source;
                if (null != ex.StackTrace) r.ErrorString += ";\r\nStackTrace=" + ex.StackTrace;
                return r;
            }
            r.ErrorCode = 0;
            r.ErrorString = "OK";
            return r;
        }

        //[WebMethod(EnableSession = true, Description = "test vložení služební cesty.")]
        //public string Test()
        //{
        //    SCOsoba cestovatel = new SCOsoba(); cestovatel.OsCislo = 13640;
        //    SCOsoba vedouci = new SCOsoba(); vedouci.OsCislo = 47;

        //    SCCesta c = new SCCesta();
        //    c.Cislo = "SC-TEST";
        //    c.Zamestnanec = cestovatel;
        //    c.Typ = "T";

        //    c.DatOd = DateTime.Now;
        //    c.PocMisto = "";
        //    c.DatDo = DateTime.Now;
        //    c.KonMisto = "";
        //    c.Cil = "Ostrava";
        //    c.Ucel = "Školení";

        //    c.DruhDopr = "AUS";
        //    c.Spz = "";

        //    c.WorkFlow = new List<SCWorkflowProc>();
        //    SCWorkflowProc p1 = new SCWorkflowProc();
        //    p1.Stav = "novy"; p1.Datum = DateTime.Now.AddDays(-2); p1.Resitel = cestovatel; p1.Duvod = "nějaký důvod";
        //    SCWorkflowProc p2 = new SCWorkflowProc();
        //    p2.Stav = "povolena"; p2.Datum = DateTime.Now.AddDays(-1); p2.Resitel = vedouci;  p2.Duvod = "";
        //    c.WorkFlow.Add(p1);
        //    c.WorkFlow.Add(p2);

        //    SCResult r = CestaVloz(c);
        //    return r.ErrorString;
        //}


        /// <summary>
        /// 
        /// </summary>
        /// <param name="s"></param>
        /// <param name="len"></param>
        /// <returns></returns>
        private string Trunc(string s, int len)
        {
            if (String.IsNullOrEmpty(s)) return s;
            if (s.Length > len) s = s.Substring(0, len);
            return s;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="o"></param>
        /// <returns></returns>
        private decimal OsobaNaOsCislo(SCOsoba o)
        {
            if (null == o) return 0;
            
            decimal osCislo;
            if (o.OsCislo != null /*&& o.OsCislo != 0*/) //PR 16.12.2014
            {
                osCislo = o.OsCislo.Value;
            }
            else
            {
               List<StrOsCisloPodnik> str_os_cisla = new List<StrOsCisloPodnik>();
               osCislo = SpravaUziv.OsCisloUzivatele(o.Login, ref str_os_cisla);                
            }
            return osCislo;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="id_poz"></param>
        /// <param name="os_cislo"></param>
        /// <param name="datum_od"></param>
        /// <param name="datum_do"></param>
        /// <param name="cil"></param>
        /// <param name="ucel"></param>
        /// <param name="druh_dopr"></param>
        /// <param name="tabulka"></param>
        /// <param name="ev_cislo"></param>
        /// <param name="vyridit_do"></param>
        public static void PozadavekInsert(ref int id_poz, decimal os_cislo, DateTime? datum_od, DateTime? datum_do, string cil, string ucel, string druh_dopr, string tabulka, int ev_cislo, DateTime? vyridit_do)
        {
            cil = InputChecker.Text(cil, "cil");
            ucel = InputChecker.Text(ucel, "ucel");
            druh_dopr = InputChecker.Text(druh_dopr, "druh_dopr");
            tabulka = InputChecker.Text(tabulka, "tabulka");

            if (datum_do == new DateTime()) datum_do = null;
            if (datum_od == new DateTime()) datum_od = null;
            if (vyridit_do == new DateTime()) vyridit_do = null;

            
            if (vyridit_do == null)
                vyridit_do = datum_od;

                                
            string ls_stav = "";
            //PR 6.8.2014 přidal jsem os_cislo
            id_poz = Pozadaveksluzc.Workfl_hl_insert("sluzc", os_cislo, "workfl_body", ref ls_stav);

            DateTime obdobi = new DateTime(((DateTime)datum_od).Year, ((DateTime)datum_od).Month, 1);
            string sql = "insert into workfl_body (id, id_poz, verze, datum_od, datum_do, " +
                                    " popis, duvod, popis2, tabulka, id_vazba, vyridit_do, obdobi, datum, zapsal)" +
                " values({0}," + id_poz + ",1," + SqlPrikaz.ValueToString(datum_od) + "," +
                        SqlPrikaz.ValueToString(datum_do) + "," +
                        SqlPrikaz.ValueToString(cil) + "," +
                        SqlPrikaz.ValueToString(ucel) + "," +
                        SqlPrikaz.ValueToString(druh_dopr) + "," +
                        SqlPrikaz.ValueToString(tabulka) + "," +
                        SqlPrikaz.ValueToString(ev_cislo) + "," +
                        SqlPrikaz.ValueToString(vyridit_do) + "," +
                        SqlPrikaz.ValueToString(obdobi) + "," +
                        SqlPrikaz.ValueToString(DateTime.Today.Date) + "," +
                        SqlPrikaz.ValueToString(0) + ")";
            //SqlPrikaz.ExecuteNonQuery(sql);
            SqlPrikaz.InsertWithSerial(sql, "workfl_body", "id");
        }

        /// <summary>
        /// Vložení záznamu do workfl_proc
        /// </summary>
        /// <param name="str"></param>
        /// <param name="sendmail"></param>
        /// <returns></returns>
        private static bool WfProcInsert(Str_workfl_proc str, bool sendmail)
        {
            int ll_exist = SqlPrikaz.ExecuteScalarInt32("select count(*) from workfl_proc " +
                                    " where id_poz=" + SqlPrikaz.ValueToString(str.id_poz) + " and stav=" + SqlPrikaz.ValueToString(str.stav) , 0);

            string s_cas = F_DateTime.Cas(str.datum);

            if (ll_exist == 0)
            {
                //SqlPrikaz.ExecuteNonQuery("update workfl_proc set archiv='a' " +
                //                        " where id_poz=" + SqlPrikaz.ValueToString(str.id_poz) + " and stav<>'diskuse'");//PR 6.8.2014 presunuto
                //string sql = "insert into workfl_proc (id, id_poz, poradi, stav, sys_stav, substav, faze, adresat, duvod, resitel, datum, cas, zapsal)" +
                //        " values (" +
                //                "{0}," +
                //                SqlPrikaz.ValueToString(str.id_poz) + "," +
                //                SqlPrikaz.ValueToString(str.poradi) + "," +
                //                SqlPrikaz.ValueToString(str.stav) + "," +
                //                SqlPrikaz.ValueToString(str.sys_stav) + "," +
                //                SqlPrikaz.ValueToString(str.substav) + "," +
                //                SqlPrikaz.ValueToString(str.faze) + "," +
                //                SqlPrikaz.ValueToString(str.adresat) + "," +
                //                SqlPrikaz.ValueToString(str.duvod) + "," +
                //                SqlPrikaz.ValueToString(str.resitel) + "," +
                //                SqlPrikaz.ValueToString(str.datum) + "," +
                //                SqlPrikaz.ValueToString(s_cas) + "," +
                //                SqlPrikaz.ValueToString(str.zapsal) +
                //            ")";
                //int id = SqlPrikaz.InsertWithSerial(sql, "workfl_proc", "id");
                //return (id>0);
                //PR 3.11.2017 
                try
                {
                    return WorkflowProces.KrokProcesu(str.id_poz, str.stav, 0, str.duvod, str.adresat, str.resitel, str.zapsal, str.datum, str.cas, sendmail);
                }
                catch (SmtpException )
                { return true; }
            }
            else
            {
                string sql = "update workfl_proc set "
                    + " resitel=" + SqlPrikaz.ValueToString(str.resitel) + ","
                    + " datum=" + SqlPrikaz.ValueToString(str.datum) + ","
                    + " cas=" + SqlPrikaz.ValueToString(s_cas)
                    + " where id_poz="+SqlPrikaz.ValueToString(str.id_poz) 
                    + " and stav=" + SqlPrikaz.ValueToString(str.stav);
                SqlPrikaz.ExecuteNonQuery(sql);
                return false;
            }
        }
    }

    /// <summary>
    /// Výsledek volání metody
    /// </summary>
    public class SCResult
    {
        /// <summary>
        /// záporná hodnota = chyba, 0 = úspěch, kladná hodnota = warning
        /// </summary>
        public int ErrorCode;
        /// <summary>
        /// doplňující text
        /// </summary>
        public string ErrorString; 
    }

    /// <summary>
    /// Struktura pro určení uživatele přes osobní číslo nebo login
    /// </summary>
    public class SCOsoba
    {
        public decimal? OsCislo;
        public string Login;
    }

    /// <summary>
    /// Záloha
    /// </summary>
    public class SCZaloha
    {
        /// <summary>
        /// částka (kladná hodnota)
        /// </summary>
        public decimal Castka;
        /// <summary>
        /// Kód měny (CZK, EUR)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string KodMeny;
        /// <summary>
        /// účel zálohy (kód z číselníku druhy cestovních náhrad)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Druh;
        /// <summary>
        /// kdo požádal o zálohu
        /// </summary>
        public SCOsoba Pozadal;
        public DateTime? PozadalDne;
        /// <summary>
        /// kdo schválil zálohu
        /// </summary>
        public SCOsoba Schvalil;
        public DateTime? SchvalilDne;
        /// <summary>
        /// kdo vyplatil zálohu
        /// </summary>
        public SCOsoba Vyplatil;
        /// <summary>
        /// datum kdy byla záloha vyplacena !!!
        /// </summary>
        public DateTime? VyplatilDne;
        /// <summary>
        /// Způsob vyplacení (kód z číselníku způsob platby)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string VyplJak;
        /// <summary>
        /// číslo dokladu
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string VyplDoklad;
    }

    public class SCPrubeh
    {
        public DateTime DatOd;
        [XmlElementAttribute(IsNullable = true)]
        public string MistoOd;
        [XmlElementAttribute(IsNullable = true)]
        public string ZemeOd;
        
        public DateTime? HraniceDat;
        [XmlElementAttribute(IsNullable = true)]
        public string HraniceMisto;

        public DateTime DatDo;
        [XmlElementAttribute(IsNullable = true)]
        public string MistoDo;
        [XmlElementAttribute(IsNullable = true)]
        public string ZemeDo;

        [XmlElementAttribute(IsNullable = true)]
        public string DruhDopr;

        public int KmMesto, KmMimo, KmKomb, KmCelkem;

        public DateTime? PracVykonOd, PracVykonDo;
        public bool? Preruseni;

    }

    public class SCVydaj
    {
        public DateTime Datum;
        public decimal Castka;
        [XmlElementAttribute(IsNullable = true)]
        public string KodMeny;
        [XmlElementAttribute(IsNullable = true)]
        public string Druh;
        [XmlElementAttribute(IsNullable = true)]
        public string ZpusPlat;
    }

    /// <summary>
    /// Proces řešení
    /// </summary>
    public class SCWorkflowProc
    {
        /// <summary>
        /// Stav (ne popis)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Stav;
        /// <summary>
        /// Datum a čas změny stavu
        /// </summary>
        public DateTime Datum;
        /// <summary>
        /// Kdo změnil stav
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public SCOsoba Resitel;
        /// <summary>
        /// Důvod (např důvod neschválení zálohy)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Duvod;
    }

    public class SCCesta
    {
        [XmlElementAttribute(IsNullable = true)]
        public string Cislo;
        [XmlElementAttribute(IsNullable = true)]         
        public SCOsoba Zamestnanec;
        [XmlElementAttribute(IsNullable = true)]
        public string Typ;
        //[XmlElementAttribute(IsNullable = true)]
        public string PocMisto;
        public DateTime DatOd;        
        //[XmlElementAttribute(IsNullable = true)]
        public string KonMisto;
        public DateTime DatDo;
        [XmlElementAttribute(IsNullable = true)]        
        public string Cil;
        [XmlElementAttribute(IsNullable = true)]
        public string Ucel;
        [XmlElementAttribute(IsNullable = true)]       
        public string DruhDopr;        
        public string Spz;
        public SCOsoba Vytvoril;
        public DateTime? VytvorDne;        
        public DateTime? DatumSouhlasu;
        public SCOsoba Povolil;        
        public DateTime? PovolilDne;

        public List<SCOsoba> Spolucestujici;
        public List<SCZaloha> Zalohy;
        public List<SCPrubeh> Prubeh;
        public List<SCVydaj> Vydaje;
        public List<SCWorkflowProc> WorkFlow;
    }
}
