﻿using KSProgram.Data;
using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using KSProgram;
using KSProgram.Extensions;
using Telerik.Web.UI;
using KSProgram.WebControls;
using KSProgram.Prenosy.SAP.CMSS.DataGroups;
using KSProgram.Funkce;
using KSProgram.Dochazka;
using static KSProgram.Dochazka.Dch_Casy;
using KSProgram.Obecne;
using KSProgram.ZakSpec;
using Microsoft.VisualBasic.FileIO;
using System.IO;

namespace KSPortal.ZakSpec
{
    public partial class DochImport : TPage
    {
        int cislo_radku = 0;
        const string _import_doch_log_key = "import_doch_log";

        public DochImport()
        {
            IdPrava = 371;
        }

        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);
            if (!IsPostBack)
            {
                Label_nadpis.Text = "Import do docházky " + Globals.AktObdobiDochazka.ToString("M/ yyyy");
                _nacistDavky();
            }
            grid_log.Visible = Zakaznik.IsCurrent("TIETO");
        }

        private void _nacistDavky()
        {
            combo_davky.Items.Clear();

            string sql = "select distinct n_pole1 from dch_casy where cas between @od and @do and n_pole1 is not null";
            ParametryPrikazu par = new ParametryPrikazu();
            par.Add(new KSParameter("od", DbType.String, Globals.AktObdobiDochazka.ToString("yyyy-MM-dd 000000")));
            par.Add(new KSParameter("do", DbType.String, Globals.AktObdobiDochazka.PosledniDen().ToString("yyyy-MM-dd 235959")));

            using (IDataReader reader = SqlPrikaz.ExecuteReader(sql, par))
            {
                while (reader.Read())
                {
                    int davka = GetNullOrData.GetInt32(reader, 0, 0);
                    combo_davky.Items.Add(davka.ToString());
                }
            }
        }

        private void _deleteDavku(int davka)
        {
            ParametryPrikazu p = new ParametryPrikazu() {
                new KSParameter("davka", DbType.Int32, davka),
                new KSParameter("zap_davka", DbType.Int32, davka * -1),
                new KSParameter("od", DbType.String, Globals.AktObdobiDochazka.ToString("yyyy-MM-dd 000000")),
                new KSParameter("do", DbType.String, Globals.AktObdobiDochazka.PosledniDen().ToString("yyyy-MM-dd 235959"))
                };
            SqlPrikaz.ExecuteNonQuery("delete from dch_casy where (n_pole1=@davka or n_pole1=@zap_davka) and cas between @od and @do", true, p);

            if (Zakaznik.IsCurrent("TIETO"))//PH 13.9.2022 používám rozhraní původně pro tieto, ale teď to bude i pro ASTUR_QANTO
            {
                p = new ParametryPrikazu() {
                new KSParameter("davka", DbType.Int32, davka),
                new KSParameter("datum", DbType.Date, Globals.AktObdobiDochazka)
                };
                SqlPrikaz.ExecuteNonQuery("delete from prtl_prescasy where obdobi=@datum and davka=@davka and co=1", true, p);
            }
        }

        private bool _davkaPouzita(int davka)
        {
            string sql = "select count(*) from dch_casy where cas between @od and @do and n_pole1=@davka";
            ParametryPrikazu par = new ParametryPrikazu();
            par.Add(new KSParameter("od", DbType.String, Globals.AktObdobiDochazka.ToString("yyyy-MM-dd 000000")));
            par.Add(new KSParameter("do", DbType.String, Globals.AktObdobiDochazka.PosledniDen().ToString("yyyy-MM-dd 235959")));
            par.Add(new KSParameter("davka", DbType.Int32, davka));
            int pocet = SqlPrikaz.ExecuteScalar<Int32>(sql, 0, ref par);

            return (pocet > 0);
        }

        protected void RadAsyncUpload1_FileUploaded(object sender, FileUploadedEventArgs e)
        {
            if (e.IsValid)
            {
                UploadedFile file = e.File;
                byte[] bytes = new byte[file.ContentLength];
                file.InputStream.Read(bytes, 0, (int)file.ContentLength);
                string str = System.Text.Encoding.GetEncoding("windows-1250").GetString(bytes);

                if (Zakaznik.IsCurrent("TIETO"))
                {
                    _import(str);
                    grid_log.Rebind();
                }
                else if(Zakaznik.IsCurrent("ASTUR_QANTO"))
                {
                    _importAsturQanto(str);
                }
                MessageBox.Alert(this, "Soubor úspěšně importován");
            }
            else
            {
                //isValid = false;
            }
            RadAsyncUpload raupload = (RadAsyncUpload)sender;
            raupload.Localization.Select = WebControlHelper.ZmenaTextu("[%Vybrat%]");
            raupload.Localization.Remove = WebControlHelper.ZmenaTextu("[%Smazat%]");
            raupload.Localization.DropZone = WebControlHelper.ZmenaTextu("[%Přetáhnout soubor sem%]");
            raupload.Localization.Cancel = WebControlHelper.ZmenaTextu("[%Zrušit%]");
        }
        private void _importAsturQanto(string str)
        {
            DateTime akt_obdobi = Globals.AktObdobiDochazka;
            DateTime posl_den = akt_obdobi.PosledniDen();

            int cislo_davky = Convert.ToInt32(rntb_davka.Value);
            _deleteDavku(cislo_davky);
            _cloarLog();

            KSProgram.Prenosy.Qanto.ImportDochazkaQanto qanto = new KSProgram.Prenosy.Qanto.ImportDochazkaQanto(akt_obdobi, cislo_davky, false);
            qanto.TestNastaveni();
            qanto.GetZaznamyFromUpload(str);
            qanto.UlozZaznamy();            

            //konec zpracování
            _nacistDavky();
        }

        

        private void _import(string str)
        {
            DateTime akt_obdobi = Globals.AktObdobiDochazka;
            DateTime posl_den = akt_obdobi.PosledniDen();

            int cislo_davky = Convert.ToInt32(rntb_davka.Value);

            _deleteDavku(cislo_davky);

            _cloarLog();

            //tady si ukládám střediska abych je neselectil furt dokola
            List<Tuple<decimal, short, string>> list_stred = new List<Tuple<decimal, short, string>>();

           // List<string> lines = str.SplitByLine().ToList();
            List<Dch_c> data_dch = new List<Dch_c>();

            //foreach (string s in lines)
           // string[] delimiters = { ";" };

            using (TextReader sr = new StringReader(str))
            {
                using (TextFieldParser parser = new TextFieldParser(sr))
                {
                    parser.Delimiters = new string[1] { ";" };
                    // Process the file's lines.
                    while (!parser.EndOfData)
                    {                        
                        // if (cislo_radku > 2000) break; //TODO: jen pro test
                        cislo_radku++;
                        string[] polozky = parser.ReadFields();
                        if (cislo_radku == 1) continue; //přeskakuju hlavičku

                        string ls_chyba = string.Empty;

                        /*
                        string s2 = s.Replace("\"", "");
                        string[] polozky = s2.Split(new char[] { ';' })
                        */
                        

                        

                        /*
                        sloupec 2 - os číslo
                        sloupec 12 - ITEM_DATE(datum)
                        sloupec 21 - kod
                        sloupec 22 - QUANTITY(doba v
                        */

                        decimal ldc_os_cislo = 0;
                        decimal ldc_pom;
                        if (decimal.TryParse(polozky[1], out ldc_pom))
                            ldc_os_cislo = ldc_pom;



                        string ls_dat = polozky[11];
                        string[] arr = ls_dat.Split('-');

                        // DateTime datum = new DateTime(int.Parse(arr[2]), int.Parse(arr[1]), int.Parse(arr[0]));
                        DateTime datum = new DateTime(int.Parse(arr[2]), int.Parse(arr[1]), int.Parse(arr[0]));
                        if (!(datum <= posl_den && datum >= akt_obdobi))
                        {
                            _log($"Datum nespadá do zpracovávaného období", ldc_os_cislo, datum);
                            continue;
                        }


                        //docházkový kod
                        //-------------------------------------
                        int kod = 0;
                        int pom_kod;
                        if (int.TryParse(polozky[20], out pom_kod))
                        {
                            kod = pom_kod;

                            if ((!Globals.DchKody.Polozky.Exists(aa => aa.Vyr_kod == kod)) && (kod != 130))
                            {
                                _log($"Neplatný Expenditure Type. Docházkový kód: {kod}, není naveden v číselníku docházkových kódů", ldc_os_cislo, datum);
                                continue;
                            }
                        }
                        else
                        {
                            _log("Nevyplněn docházkový kód", ldc_os_cislo, datum);
                            continue;
                        }

                        //-------------------------------------

                        decimal? hodiny = F_String.StrDecNum(polozky[21]);
                        if ((hodiny == null) || (hodiny == 0))
                        {
                            _log("Nulové nebo nevyplněny hodiny", ldc_os_cislo, datum);
                            continue;
                        }
                        /*
                        string ls_doba=F_DateTime.DobaDecNaDobaString((decimal)Math.Abs((decimal)hodiny));
                        if (hodiny < 0) ls_doba = "-" + ls_doba;
                        */

                        //číslo poměru
                        //---------------------------------------
                        short cislo_pom = -1;
                        short pomer;
                        if (Prac_pom.Platny_Pomer_Den(ldc_os_cislo, datum, out pomer))
                            cislo_pom = pomer;
                        else
                        {
                            _log("Datum mimo PV", ldc_os_cislo, datum);
                            continue;
                        }
                        //---------------------------------------

                        //pracovní středisko
                        //---------------------------------------
                        string prac_str = list_stred.Where(aa => aa.Item1 == ldc_os_cislo && aa.Item2 == cislo_pom).Select(aa => aa.Item3).FirstOrDefault();
                        if (string.IsNullOrEmpty(prac_str))
                        {
                            prac_str = Prac_pom.Get_prac_str(ldc_os_cislo, cislo_pom);
                            if (!string.IsNullOrEmpty(prac_str))
                            {
                                Tuple<decimal, short, string> it = new Tuple<decimal, short, string>(ldc_os_cislo, cislo_pom, prac_str);
                                list_stred.Add(it);
                            }
                        }
                        if (string.IsNullOrEmpty(prac_str))
                        {
                            _log("Nenalezeno pracovní středisko", ldc_os_cislo, datum);
                            continue;
                        }
                        //-------------------------------------------

                        if (kod == 130)
                        {
                            int delka = Convert.ToInt32((decimal)hodiny * 60);
                            Prtl_prescasy.InsertPrescasy(ldc_os_cislo, cislo_pom, delka, prac_str, datum.Date, akt_obdobi, cislo_davky);
                        }
                        else
                        {
                            Dch_c it = new Dch_c();
                            it.OsCislo = ldc_os_cislo;
                            it.CisloPom = cislo_pom;
                            it.Kod = kod;
                            it.Datum = datum.Date;
                            it.PracStr = prac_str;
                            //it.Minuty = (Int32)((decimal)hodiny * 60);
                            it.Hodiny = (hodiny == null ? 0 : (decimal)hodiny);

                            data_dch.Add(it);
                        }
                    }
                }
            }
            
        
    
            //musí data sesumovat,protože přichází i záporné hodiny, kterýma jsou nulované kladné hodiny           
            //sumuju raději v paměti mělo by to být rychlejší
            var dt = from s in data_dch                    
                     group s by new { s.OsCislo, s.CisloPom, s.Kod, s.PracStr, s.Datum } into g
                     orderby g.Key.OsCislo, g.Key.CisloPom
                     select new { OsCislo = g.Key.OsCislo, CisloPom = g.Key.CisloPom, g.Key.Kod, g.Key.PracStr, g.Key.Datum, Hodiny = g.Sum(ss => ss.Hodiny) };
            foreach (var c in dt)
            {
                Str_Dch_Casy d = new Str_Dch_Casy();
                d.Os_cislo = c.OsCislo;
                d.Cislo_pom = c.CisloPom;
                d.Kod = c.Kod;
                d.Prac_str = c.PracStr;
                d.Cas = c.Datum;
                d.Doba = Cas_na_min.GetCas((int)(c.Hodiny * 60));
                d.Operace = "a";
                d.N_pole1 = cislo_davky;
                Dch_Casy.DbInsert(d);
            }
            dt = null;
            //-----------------------------------------------------------------------
            data_dch.Clear();
            
            _nacistDavky();

        }

        private void _cloarLog()
        {
            SessionInfo.RemoveVariable(_import_doch_log_key);
        }

        private void _log(string mess, decimal? os_cislo, DateTime? datum)
        {
            string m= $"Řádek: {cislo_radku}, {mess}";
            ImportDochLog it = new ImportDochLog();
            it.c_radku = cislo_radku;
            it.os_cislo = os_cislo;
            it.datum = datum;
            it.zprava = mess;
            
            List<ImportDochLog> lst = new List<ImportDochLog>();
            object o= SessionInfo.GetVariable(_import_doch_log_key);
            if (o!=null)
                lst = (List<ImportDochLog>)o;
            lst.Add(it);

            SessionInfo.SetVariable(_import_doch_log_key, lst);
        }

        protected void rntb_davka_TextChanged(object sender, EventArgs e)
        {
            lab_info.Visible = false;
            double? val=((RadNumericTextBox)sender).Value;
            if (val!=null)
            {
                if (_davkaPouzita(Convert.ToInt32(val)))
                {
                    // KSProgram.WebControls.MessageBox.NotifikaceInfo($"Dávka {val}, je již použita, pokračováním dojde k přemazání dávky se stejným číslem");
                    lab_info.Visible = true;
                    lab_info.Text = $"Dávka { val}, je již použita, pokračováním dojde k přemazání dávky se stejným číslem";
                }
            }
        }

        protected void grid_log_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
        {
            List<ImportDochLog> lst = new List<ImportDochLog>();
            object o = SessionInfo.GetVariable(_import_doch_log_key);
            if (o != null)
                lst = (List<ImportDochLog>)o;

            ((RadGrid)sender).DataSource = lst;
        }

        public class ImportDochLog
        {
            public int c_radku { get; set; }

            public decimal? os_cislo { get; set; }

            public DateTime? datum { get; set; }

            public string zprava { get; set; }
        }

    }
}